/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770	*/
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566	*/
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/
/*                                                                              */
/*			IOC9010 PROJECT	--- Hytec IP Card API Library Testing Program		*/
/*                                                                              */
/********************************************************************************/
/*  Source file name      :-      FAT8515Master.c								*/
/*  Definition header file:		  HYTECIPAPI.h									*/
/*								  Hy8515.h										*/
/*  Platform			  :		  Linux, Windows XP, Windows 2000               */
/*  Initial creation date :-      25-NOV-2010                                   */
/*                                                                              */
/*  Original Developers   :-      Jim Chen	                                    */
/********************************************************************************/
/*                                                                              */
/*  Description :-                                                              */
/*                                                                              */
/*  This is a test program to try and reproduce a possible problem using the    */
/*   new API to send/receive via multiple 8515 channels                        	*/
/*                                                                              */
/*																				*/
/*  (C)2009 Hytec Electronics Ltd.												*/
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Modification record :-                                                      */
/*                                                                              */
/*  Date modified |     Person    |   Description of modification               */
/* ---------------|---------------|-------------------------------------------- */
/*    25/11/2010  |	    JC        |	  Initial copy								*/
/*                |               |                                             */
/*                                                                              */
/********************************************************************************/


#include "../include/HytecIPAPI.h"
#include "../include/Hy85156.h"

#include <pthread.h>
#include <math.h>

/* Global variables */
  
static CHy85156 COM1;

static int status;
UCHAR my_request[] = "Message form port 0.\n\a";   //20
static int port0 = 0;

int IPslot = 'A';
char vector = 0x0;				/* no interrupt */
int delay = 0;
int halfduplex = 0;
int delay485 = 0;

int vmeslot = 3;
	
ULONG baud = 9600;
char parity = 'N'; 
int stopbits = 1;
int databits = 8;
char flowctl = 0x10;
    
int reason = 0;
int times = 0;
UCHAR buf[255];

/* Global variables */

struct pvtDrv
{
    char INTVector;
    char INTLevel;
}myDrv;

/**************To simulate getch() function **********/
#include <termios.h>
#include <unistd.h>
int getsch( ) 
{
  struct termios oldt, newt;
  int            ch;
  tcgetattr( STDIN_FILENO, &oldt );
  newt = oldt;
  newt.c_lflag &= ~( ICANON | ECHO );
  tcsetattr( STDIN_FILENO, TCSANOW, &newt );
  ch = getchar();
  tcsetattr( STDIN_FILENO, TCSANOW, &oldt );
  return ch;
}
/**************To simulate getch() function **********/



void callback(void *drv)
{
	int status, intport;

	printf("\n*****************************\n");
	printf("         Interrupted         \n");
	printf("=============================\n");
//	EN_VME_INT_INFO *d = (EN_VME_INT_INFO *)drv;

//	times++;
//	printf("INT Level=%x Vector=%x vecCount=%d  INT No.=%d\n", d->intNum, d->vectors[0], d->vecCount, times);	

	status = COM1.Hy85156_INTReason(&intport, &reason);
	if (status !=0)
	{
		printf("Get intreason error. %d \n",status);
	}else
	{
		if(reason & INTERRUPT_REASON_LASER_READY)
	    	printf("Laser ready!. reason=%x port=%d\n", reason, intport);
	}
	status = COM1.Hy85156_ResetInterrupt(intport, reason);	//reset interrupt
	printf("*****************************\n\n");
}


int main(void)
{ 
	status = 0;
    int interruptLevel = 5, i, j;

	printf("\nTesting Program starts......");
	int flag, no=0, num=0;


	USHORT id_buffer[14];
	USHORT carrier; 
	USHORT CSR;

	CSR = 0;

    //ioc.IOC9010SetCarrierFanSpeed(1,1);
	status = COM1.IOC9010GetCarrier(vmeslot, &carrier);

    printf("Carrier Card is %04X\n",carrier);

        printf("Detect IP card installed.....\n\n");
        for (j=0;j<6;j++)
        {
            status = COM1.IOC9010GetConfig(vmeslot, j, 14, id_buffer);
            if (status == C_ERR_NODEVICE)
            {
                printf("Device open failed!");
                break;
            }
            else
            {
                printf("Slot %c IP identity: ", (char) (j+0x41));
                for (i=0;i<14;i++)
    				printf("%X ",id_buffer[i]);
                printf("\n");
            }
	   }



	do
	{
		printf("Please type IP slot (A ~ F):\n");
		IPslot = getsch() & 0xFF;		
		if(IPslot >0x60) IPslot=IPslot-0x20;
		if((IPslot-0x40>0) && (IPslot-0x40<6)) flag = 1;
	}while(flag == 0);


	//generic settings for the 8515 COM1
	status = COM1.Hy85156_Configure(vmeslot, IPslot, vector, delay, interruptLevel, &callback, &myDrv);
	if (status < 0)
	{
		printf("Configure 8515 for COM1 error! %d\n", (int) status);
		return 0;	
	}else
		printf("Configured OK..\n");

	//setup com1 port 0 for sending
	status = COM1.Hy85156_ComsSetup(port0, baud, parity, stopbits, databits, flowctl);
	if (status < 0)
	{
		printf("Setup COM1 error! %d\n", (int) status);
		return 0;	
	}else
		printf("Setup port%d OK.\n\n", port0);

    //COM1.Hy85156_EnableInterrupt(port0, ENABLE_MODEM_INT_85156, 0, 0);

	for (j=1; j<8; j++)
	{
		printf("Sent to port%d:%s\n", j, my_request);
		no = 0;
		for(i=0; i<5; i++)
		{
			status = COM1.Hy85156_Write(port0, my_request, strlen((char*)my_request));
			printf("Sent to port%d.\n", j);

    	    int counter = 0;
    	    num = 0;
    	    do
    	    {
    	        status = COM1.Hy85156_Read(port0, &buf[num]);
    	        if(status > 0)
    	        {
    	            num += status;
					buf[num]=0;
    	            if (strstr((const char*)buf, "\n\a") != NULL)
    	            {
    	                printf("RcvNo=%d : %s", num, buf);
	
    	                num = 0;
    	                break; 
    	            }
					//printf("received %d chars: %x\n\r", num, buf[0]);   
    	            status = 0;
    	        }else
    	        {
    	            counter++;
    	            if (counter < 0) break;
    	        }
    	    }while(status == 0);
			no++;
			if(no >= 5) break;

			usleep(1000000);

		}
		if(j <= 7)
		{
			printf("\nMove cable to port%d when ready hit any key.\n", j+1);
			getchar();		
		}
	}

    return(0);
}

