/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770	*/
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566	*/
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/
/*                                                                              */
/*			IOC9010 PROJECT	--- Hytec IP Card API Library Testing Program		*/
/*                                                                              */
/********************************************************************************/
/*  Source file name      :-      Test8516Node1.c								*/
/*  Definition header file:		  HYTECIPAPI.h									*/
/*								  Hy8516.h										*/
/*  Platform			  :		  Linux, Windows XP, Windows 2000               */
/*  Initial creation date :-      21-OCT-2009                                   */
/*                                                                              */
/*  Original Developers   :-      Jim Chen	                                    */
/********************************************************************************/
/*                                                                              */
/*  Description :-                                                              */
/*                                                                              */
/*  This is the first node of RS485 testing program                             */
/*                                                                              */
/*																				*/
/*  (C)2009 Hytec Electronics Ltd.												*/
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Modification record :-                                                      */
/*                                                                              */
/*  Date modified |     Person    |   Description of modification               */
/* ---------------|---------------|-------------------------------------------- */
/*    22/10/09    |	    JC        |	  Initial copy								*/
/*                |               |                                             */
/*                                                                              */
/********************************************************************************/


#include "../include/HytecIPAPI.h"
#include "../include/Hy85156.h"

#include <pthread.h>
#include <math.h>

/* Global variables */
   
static CHy85156 COM1;
static long status;
UCHAR response_from_slave[] = "The response from";
UCHAR sendbuf[255];

int no;

static int port = 7;

int IPslot = 'D';
char vector = 0x0;				/* no interrupt */
int delay = 0;
int halfduplex = 2;
int delay485 = 5;

int vmeslot = 0;
	
ULONG baud = 9600;
char parity = 'N'; 
int stopbits = 1;
int databits = 8;
char flowctl = 0;
int i;
/* Global variables */

void initialisePort(int portNum);

/**************To simulate getch() function **********/
#include <termios.h>
#include <unistd.h>
int getsch( ) 
{
  struct termios oldt, newt;
  int            ch;
  tcgetattr( STDIN_FILENO, &oldt );
  newt = oldt;
  newt.c_lflag &= ~( ICANON | ECHO );
  tcsetattr( STDIN_FILENO, TCSANOW, &newt );
  ch = getchar();
  tcsetattr( STDIN_FILENO, TCSANOW, &oldt );
  return ch;
}
/**************To simulate getch() function **********/

int main(void)
{ 
	status = 0;
    UCHAR buf[255];
    int num;
	printf("\nTesting Program starts......\n");
	int flag=0, startport, j;
	do
	{
		printf("Please type IP slot (A ~ F):\n");
 		IPslot=getsch() & 0xFF;
		if(IPslot >0x60) IPslot=IPslot-0x20;
		if((IPslot-0x40>0) && (IPslot-0x40<6)) flag = 1;
	}while(flag == 0);

	flag = 0;
	do
	{
		printf("Please type IP starting port number (2 ~ 8):\n");
		startport = getsch() & 0xFF;		
		if((startport-0x32>=0) && (startport-0x30<9)) 
		{
			flag = 1;
			startport = startport-0x30;
		}
	}while(flag == 0);

	printf("IP Slot: %c, start port: SK%d\n", IPslot, startport);


	COM1.IOC9010SetIPClock(IPslot, 0);

	//generic settings for the 8515 COM1
	status = COM1.Hy85156_Configure(vmeslot, IPslot, vector, delay);
	if (status < 0)
	{
		printf("Configure 8516 error! %d\n", (int) status);
		return 0;	
	}else
		printf("Configured slot %c OK.\n", IPslot);
    
	for(port=startport; port<9; port++)
	{
	    initialisePort(port-1);
        printf("Waiting message from SK1...port=%d\n\r", port-1);
//getchar();
    	/* main forever loop, doing nothing */
    	num = 0;
    	buf[0] = 0;
		no = 0;
		do
		{
    	    status = COM1.Hy85156_Read(port-1, &buf[num]);
//getchar();
			if(status > 0)
			{
    	        num += status;
				buf[num] = 0;
    	        //printf("status = %d\n", status);
    	        if (strstr((const char*)buf, "\n\a") != NULL)
    	        {
    	            printf("Received %d chars: %s", num, buf);
    	            usleep(10000);
	
    	            sprintf((char*)sendbuf, "%s SK%d.\n\a", response_from_slave, port);
    	            status = COM1.Hy85156_Write(port-1, sendbuf, strlen((char*)sendbuf));
    	            num = 0;
					no++;
					if(no >= 5) break;
    	        }
				else
				{
/*					printf("partial =");
					for(j=0; j<num; j++)
					{
    	            	printf("%x ", buf[j]);                
					}
					printf(" n=%d\n\a", num);*/
				}
    	    }
			usleep(2000);
		}while(1);
/*		if(port<9)
		{
			printf("\nPlease move cable to SK%d and hit return.\n",port+1);
			getchar();
		}*/
	}
    return(0);
}

/* The thread that does the thing */
void initialisePort(int portNum)
{

	//setup com1 port 0 for sending
	status = COM1.Hy85156_ComsSetup(portNum, baud, parity, stopbits, databits, flowctl, halfduplex, delay485);
	if (status < 0)
	{
		printf("\nSetup SK%d error! %d\n", portNum+1, (int) status);
		return;	
	}else
		printf("\nSetup SK%d OK.\n", portNum+1);


}
		
