/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770	*/
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566	*/
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/
/*                                                                              */
/*			IOC9010 PROJECT	--- Hytec IP Card API Library Testing Program		*/
/*                                                                              */
/********************************************************************************/
/*  Source file name      :-      Test8506.c									*/
/*  Definition header file:		  HYTECIPAPI.h									*/
/*								  Hy85056.h										*/
/*  Platform			  :		  Linux, Windows XP, Windows 2000               */
/*  Initial creation date :-      25-NOV-2008                                   */
/*                                                                              */
/*  Original Developers   :-      Mark Woodward                                 */
/********************************************************************************/
/*                                                                              */
/*  Description :-                                                              */
/*                                                                              */
/*  This is a test program to try and reproduce a possible problem between the  */
/*  3331 controller and 8506 ip cards.                                       	*/
/*                                                                              */
/*																				*/
/*  (C)2008 Hytec Electronics Ltd.												*/
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Modification record :-                                                      */
/*                                                                              */
/*  Date modified |     Person    |   Description of modification               */
/* ---------------|---------------|-------------------------------------------- */
/*    25/11/08    |	    MW        |	  Initial copy								*/
/*                |               |                                             */
/*                                                                              */
/********************************************************************************/


#include "../include/HytecIPAPI.h"
#include "../include/Hy8512.h"

#include <pthread.h>
#include <math.h>
#include <sys/time.h>


int vmeslot = 2;        //ignored by IOC9010

int main(void)
{
	printf("\nIOC9010 floating calculation speed testing starts...... \n\n");

	CHy8512 SCALER1;
    char scalerIPslot1     = 'A';
    USHORT scaler_arm   = 0xFFFF;   // Arm / Enable all Scalers
    USHORT clock_source = 0xFFFF;   // Clock Internal
    ULONG scalers[16];
	USHORT myRegVal;
	int status;
	int times = 10000;

    // Test 1 - Read All Scalers at Once at 1 Hz Clocked from Internal 10MHz Clock
    // --------------------------------------------------------------------------- 
	SCALER1.IOC9010SetIPClock(0, 1);		//set 32MHz
	status = SCALER1.Hy8512_Configure(vmeslot, scalerIPslot1, scaler_arm, clock_source, 0, 0);

  	int loop;
 
  	/* simulate 5 KHz */
  	timespec ts1, ts2;
  	clock_gettime(CLOCK_REALTIME, &ts1);

  	for (loop=0; loop<times; loop++) { 
    	status = SCALER1.Hy8512_Read(scalers, 0);
  	}
  	clock_gettime(CLOCK_REALTIME, &ts2);

	printf("Before: sec.nanosec=%d.%d\n", ts1.tv_sec,ts1.tv_nsec);
  	printf("After: sec.nanosec=%d.%d\n", ts2.tv_sec,ts2.tv_nsec);
	if(ts2.tv_nsec < ts1.tv_nsec)
	{
		ts2.tv_sec--;
		ts2.tv_nsec += 1000000000;
	}
  	long int sdiff = ts2.tv_sec - ts1.tv_sec;
  	long int ndiff = ts2.tv_nsec - ts1.tv_nsec;
  	double diff = (double)sdiff * 1000000000 + (double)ndiff;
  	printf("Total time taken for %d times: %f ns\n", times, diff);
  	printf("Time taken: %f ns/loop/channel/word\n", (double)diff/times/16/2);

    return(0);
}
