/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770	*/
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566	*/
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/
/*                                                                              */
/*			IOC9010 PROJECT	--- Hytec IP Card API Library Testing Program		*/
/*                                                                              */
/********************************************************************************/
/*  Source file name      :-      TestInterrupt.c								*/
/*  Definition header file:		  HYTECIPAPI.h									*/
/*								  Hy85056.h										*/
/*								  Hy.....h										*/
/*  Platform			  :		  Linux, Windows XP, Windows 2000               */
/*  Initial creation date :-      26-MAR-2009                                   */
/*                                                                              */
/*  Original Developers   :-      Jim S Chen	                                */
/********************************************************************************/
/*                                                                              */
/*  Description :-                                                              */
/*                                                                              */
/*  This program tests interrupts for IOC9010, Hytec 5331/5332 with VME64x 		*/
/*  and Hytec PCI express 6335 carrier cards                                   	*/
/*                                                                              */
/*																				*/
/*  (C)2009 Hytec Electronics Ltd.												*/
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Modification record :-                                                      */
/*                                                                              */
/*  Date modified |     Person    |   Description of modification               */
/* ---------------|---------------|-------------------------------------------- */
/*    26/03/09    |	    JC        |	  Initial copy								*/
/*                |               |                                             */
/*                                                                              */
/********************************************************************************/


#include "../include/HytecIPAPI.h"
#include "../include/Hy8512.h"

#include <pthread.h>


#include <math.h>

/******** Global variable section ********/

struct pvtDrv
{
    char INTVector;
    char INTLevel;
};

pvtDrv myDrv;


//application dependant variables
CHy8512 SCALER1;
long status;
long number;
int port_send = 4, port_rcv = 0;
UCHAR my_word[] = "The sharper than expected fall in economic output in the first quarter of this year does not alter the government's forecast for a recovery by the end of the year. We hoped...\n\a";
UCHAR word[64];

/******End of Global variable section ******/

void callback(void *drv)
{
	int i, status;
	USHORT intreason;

	printf("\n*****************************\n");
	printf("         Interrupted         \n");
	printf("=============================\n");
	pvtDrv *d = (pvtDrv *)drv;

	printf("vector=%x\n", d->INTVector);	

	SCALER1.Hy8512_INTReason(&intreason);
	status = SCALER1.Hy8512_ResetInterrupt(intreason);
	if (status !=0)
	{
		printf("Get intreason error. %x \n",status);
	}
	printf("*****************************\n\n");
}

int main(void)
{
     //configuration data for ADC
    int vmeslot = 0;
	char scalerIPslot1     = 'A';
    USHORT scaler_arm   = 0xFFFF;   // Arm / Enable all Scalers
    USHORT clock_source = 0x0000;   // Clock External
    ULONG scalers[16];
	USHORT myRegVal;
	int i, j;

    // Test 1 - Read All Scalers at Once at 1 Hz Clocked from Internal 10MHz Clock
    // --------------------------------------------------------------------------- 

    clock_source = 0xFFFF;          // Clock Internal 10MHz
	status = SCALER1.Hy8512_Configure(vmeslot, scalerIPslot1, scaler_arm,
										clock_source, 0x44, 0xFFFF, 1, &callback, &myDrv);

	if (status != 0)
	{
		printf("Driver error!\n");
		return 0;	
	}
    else
    {
		printf("Configure IP card successfully. Return value = %d \n",(int)status);
    }

	SCALER1.Hy8512_EnableInterrupt();
	SCALER1.Hy8512_ArmEnable(0xFFFF);
	SCALER1.Hy8512_OverflowClear(0xFFFF);


	status = SCALER1.Hy8512_Read_Reg(0, &myRegVal);
	printf("Reg 0 = %x  %d\n",(int)myRegVal,(int)status);


    for (i = 0; i < 450; i++)
	{
	    // Read all Scalers Clearing Each Time
        status = SCALER1.Hy8512_Read(scalers, 0);
	status = SCALER1.Hy8512_Read_Reg(0, &myRegVal);
		if (status == 0)
		{
    		for(j = 0; j < 1;j++)
    		{	
				printf("%d. Channel %d value = %d CSR: %x\n", i, j, (scalers[j]), myRegVal );
			}
		} 
        else
    	{
			printf("Read channel %d error! %d \n", j, (int)status);
			return 0;
		}
        // Once a second
        sleep(1);
    }  
	status = SCALER1.Hy8512_Read_Reg(0, &myRegVal);
	printf("Reg 0 = %x  %d\n",(int)myRegVal,(int)status);

getchar(); 

	return 0;

}
