/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770	*/
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566	*/
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/
/*                                                                              */
/*				IOC9010 PROJECT	--- Hytec IP Card API Functions 				*/
/*							for Windows and Linux Platforms	    				*/
/*                                IP Card Type: 8005                           */
/********************************************************************************/
/*  Source file name      :-      Hy8005.h										*/
/*  Definition header file:		  HYTECIPAPI.h									*/
/*  Platform			  :		  Linux, Windows XP, Windows 2000               */
/*  Initial creation date :-      20-JUN-2008                                   */
/*                                                                              */
/*  Original Developers   :-      DAN & MW                                      */
/********************************************************************************/
/*                                                                              */
/*  Description :-                                                              */
/*                                                                              */
/*  This header file is designed for Hytec API function library of 8005         */
/*	carrier card. It defines the class prototype, error code, masks and 		*/
/*	registers of 8005													        */
/*																				*/
/*  The design of this interface is cross platforms, which means the code 		*/
/*	can be used in operating systems like Windows, Linux etc without changes.	*/
/*                                                                              */
/*  The language used is C++ but the final product -- the Dynamic Link library	*/
/*	(dll) or the shared library can be used by either C or C++ applications.	*/
/*																				*/
/*  (C)2008 Hytec Electronics Ltd.												*/
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Modification record :-                                                      */
/*                                                                              */
/*  Date modified |     Person    |   Description of modification | Version     */
/* ---------------|---------------|-------------------------------------------- */
/*    20/06/08    |	  DAN + MW    |	  Initial copy				  |	0.0.1		*/
/*                |               |                                             */
/*                                                                              */
/********************************************************************************/
#include "HytecIPAPI.h"

#if PLATFORM == 1

#define HYTECAPI_EXPORTS 1
#ifdef HYTECAPI_EXPORTS
#define HYTECAPI_API __declspec(dllexport)
#else
#define HYTECAPI_API __declspec(dllimport)
#endif
#include "C:\Hytec\IOC9010\IOCGeneric\IOCGeneric\IOCGeneric.h"

#else

#include "IOCGeneric.h"

#endif



//error code definition
#define NOT_8005 -8505010
#define Hy8005_NOT_CONFIGURED -8005012
#define DEBOUNCE_RATE_INVALID_8005 -8005013
#define SCAN_RATE_INVALID_8005 -8005014
#define DIRECTION_INVALID_8005 -8005015
#define PULSE_WIDTH_INVALID_8005 -8005016
#define INHIBIT_SETTING_INVALID_8005 -8005017
#define PORT_NUMBER_INVALID_8005 -8005018
#define WRITE_VIOLATION_8005 -8005019
#define BIT_NUMBER_INVALID_8005 -8005020

//signature definition
#define SIGNATURE_8005 0x8005

//register address defines
#define DATA_8005 0x0000
#define CSR_8005 0x0004
#define INTERRUPT_MASK_8005 0x0008
#define DEBOUNCE_MASK_8005 0x000C
#define PULSE_MASK_8005 0x0010
#define PULSE_SETTING_8005 0x0014
#define FIFO_REG_8005 0x0018
#define FIFO_MEM_8005 0x001C
#define INTERRUPT_VECTOR_8005 0x0020

//mask definition
#define DEBOUNCE_BIT_MASK_8005 0xFFE7
#define SCANRATE_BIT_MASK_8005 0xFF9F
#define DIR_BIT_MASK_8005 0xF3FF
#define INHIBIT_BIT_MASK_8005 0xFFF9
#define ENABLE_SCAN_8005 0x0001

#define SIGNAL_INVERT 0x0004

// This class is exported from the HYTECAPI.dll
#if PLATFORM == 1									//windows
class HYTECAPI_API CHy8005 : public CIOCGeneric
#else											//linux
class CHy8005 : public CIOCGeneric
#endif
{
private:
	long m_ErrorCode;							// global error code holder
	int initIPSlot;								// remember the IP slot when configures
	int initVMESlot;
    int Hy8005_invert;                                 // controls bit inversion for i/o
                                                //  1 = inverted 

public:
	CHy8005(void);
	CHy8005(int vmeslot);
	~CHy8005(void);

	long GetErrorCode()
	{
		return m_ErrorCode;
	}

/*----------------------------------------------------------------------------------*/
/*Function:		Configure 8005 IP card												*/
/*----------------------------------------------------------------------------------*/
/*Description:	8005 IP configuration routine										*/
/*				according to the settings.											*/
/*Prototype:	long Hy8005_Configure(int vmeslot, int debrate,						*/
/*				ULONG debmask, int scanrate, int dir, USHORT vectornum,				*/
/*				ULONG intmask,	int pwidth, ULONG pmask, int inhibit)				*/
/*Arguments:	vmeslot = VME slot number											*/
/*				ipslot = -dummy arg - ignored - always zero							*/
/*				portnum = port number, 0-3											*/
/*				debrate = Debounce Rate (0=100Hz, 1=200Hz, 2=500Hz, 3=1kHz)			*/
/*				debmask = Debounce Bit Mask, 0x00000000-0xFFFFFFFF					*/
/*				scanrate = input scan rate (0=1kHz, 1=10kHz, 2=100kHz, 3=1MHz)		*/
/*				dir = The direction of the I/O										*/
/*						0 = All 32 bits inputs.										*/
/*						1 = All 16 bits outputs.									*/
/*				vectornum = Interrupt Vector for this IP Card (= 0, find me one).	*/
/*				intmask = Interrupt Bit Mask, 0x0000 - 0xFFFFFFFF.					*/
/*						0x0000 means none											*/
/*				pwidth = pulse width, 0 ~ 9	see nmanual								*/
/*				pmask = Pulse Output Bit Mask, 0x0000 - 0xFFFFFFFF					*/
/*				inhibit = 0 = front panel control									*/
/*						  1 = software control										*/
/*						  2 = disabled												*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
long Hy8005_Configure(int vmeslot, char ipslot, int portnum, int debrate, 
								 ULONG debmask, int scanrate, int dir, 
								 USHORT vectornum, ULONG intmask,
								 int pwidth, ULONG pmask, int inhibit);

/*----------------------------------------------------------------------------------*/
/*Function:		Read status	by word													*/
/*----------------------------------------------------------------------------------*/
/*Description:	Read the last status from the data register of 8005 carrier board	*/
/*Prototype:	long Hy8005_Read(int portnum, ULONG *value, ULONG mask)				*/
/*Arguments:	portnum = number of port (0-3) to read								*/
/*				*value = pointer to the 16bit value variable which holds the return	*/
/*				mask = bit mask for the read										*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
long Hy8005_Read(int portnum, ULONG *value, ULONG mask);

/*----------------------------------------------------------------------------------*/
/*Function:		Read status	by bit													*/
/*----------------------------------------------------------------------------------*/
/*Description:	Read one bit of the last status register of 8005 IP card			*/
/*Prototype:	bool Hy80054_Read_Bit(int Bit)										*/
/*Arguments:	portnum = number of port (0-3) to read								*/
/*				Bit = bit to read, 0 ~ 31											*/
/*Return:		true if bit = 1, false if bit = 0.									*/
/*----------------------------------------------------------------------------------*/
bool Hy8005_Read_Bit(int portnum, int Bit);

/*----------------------------------------------------------------------------------*/
/*Function:		Write output by word for 8005										*/
/*----------------------------------------------------------------------------------*/
/*Description:	Write data to the data register of 8005 Carrier card				*/
/*Prototype:	long Hy85056_Write(int portnum, ULONG mask, ULONG value)			*/
/*Arguments:	portnum = number of port (0-3) to read								*/
/*				mask = bit mask for the write										*/
/*				*value = pointer to the 32bit value variable which holds the data	*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
long Hy8005_Write(int portnum, ULONG mask, ULONG value);

/*----------------------------------------------------------------------------------*/
/*Function:		Write one bit to 8005												*/
/*----------------------------------------------------------------------------------*/
/*Description:	Write one bit to 8005 data register									*/
/*Prototype:	long Hy8005_Write_Bit(int portnum, int Bit, ULONG value)			*/
/*Arguments:	portnum = number of port (0-3) to read								*/
/*				Bit = bit to write = 0 ~ 15											*/
/*				value = 1 to set, 0 to clear										*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
long Hy8005_Write_Bit(int portnum, int Bit, ULONG value);

};

#if PLATFORM == 1									//windows
extern "C" HYTECAPI_API CHy8005* CreateCHy8005();
extern "C" HYTECAPI_API void DeleteCHy8005( CHy8005* instance );
#else											//linux
extern "C" CHy8005* CreateCHy8005();
extern "C" void DeleteCHy8005( CHy8005* instance );
#endif
