/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770	*/
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566	*/
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/
/*                                                                              */
/*				IOC9010 PROJECT	--- Hytec IP Card API Functions 				*/
/*							for Windows and Linux Platforms	    				*/
/*                                IP Card Type: 8402                            */
/********************************************************************************/
/*  Source file name      :-      Hy8402.h										*/
/*  Definition header file:		  HYTECIPAPI.h									*/
/*  Platform			  :		  Linux, Windows XP, Windows 2000               */
/*  Initial creation date :-      11-AUG-2006                                   */
/*                                                                              */
/*  Original Developers   :-      Jim S Chen                                    */
/********************************************************************************/
/*                                                                              */
/*  Description :-                                                              */
/*                                                                              */
/*  This header file is designed for Hytec API function library of 8402 IP card.*/
/*  It defines the class prototype, error code, masks and registers of 8402.    */
/*																				*/
/*  The design of this interface is cross platforms, which means the code 		*/
/*	can be used in operating systems like Windows, Linux etc without changes.	*/
/*                                                                              */
/*  The language used is C++ but the final product -- the Dynamic Link library	*/
/*	(dll) or the shared library can be used by either C or C++ applications.	*/
/*																				*/
/*  (C)2006 Hytec Electronics Ltd.												*/
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Modification record :-                                                      */
/*                                                                              */
/*  Date modified |     Person    |   Description of modification | Version     */
/* ---------------|---------------|-------------------------------------------- */
/*    11/08/06    |	  Jim S Chen  |	  Initial copy				  |	0.0.1		*/
/*                |               |                                             */
/*                                                                              */
/********************************************************************************/
#include "HytecIPAPI.h"

#if PLATFORM == 1

#define HYTECAPI_EXPORTS 1
#ifdef HYTECAPI_EXPORTS
#define HYTECAPI_API __declspec(dllexport)
#else
#define HYTECAPI_API __declspec(dllimport)
#endif
#include "C:\Hytec\IOC9010\IOCGeneric\IOCGeneric\IOCGeneric.h"

#else

#include "IOCGeneric.h"

#endif


//error code definition
#define NOT_8402_IP -8402001
#define CLOCKRATE_INVALID_8402 -8402012
#define CHANNEL_INVALID_8402 -8402013
#define Hy8402_NOT_CONFIGURED -8402014

//mask definition
#define DORAM 0x4000
#define CLOCKSOURCE 0x1000
#define INHIBIT 0x0800
#define ARMDAC 0x8000

//signature definition
#define SIGNATURE_8402 0x8402

//register address defines
#define CSR_8402 0x0000
#define CLOCK_RATE_8402 0x0006
#define FIRST_CHANNEL_8402 0x0010
#define CHANNEL_MEMORY_BLOCK_8402 0x08000

// This class is exported from the HYTECAPI.dll
#if PLATFORM == 1									//windows
class HYTECAPI_API CHy8402 : public CIOCGeneric
#else											//linux
class CHy8402 : public CIOCGeneric
#endif
{
private:
	long m_ErrorCode;							// global error code holder
	int initIPSlot;								// remember the IP slot when configures
	int initVMESlot;

public:
	CHy8402(void);
	CHy8402(int vmeslot, char ipslot);
	~CHy8402(void);

	long GetErrorCode()
	{
		return m_ErrorCode;
	}
	
/*----------------------------------------------------------------------------------*/
/*Function:		Configure 8402 IP card												*/
/*----------------------------------------------------------------------------------*/
/*Description:	8402 IP configuration routine										*/
/*				according to the settings.											*/
/*Prototype:	long Hy8402_Configure(int vmeslot, char ipslot, int vectornum,		*/
/*				int doram, int clockSource, int clockRate, int inhibit)				*/
/*Arguements:	vmeslot = VME slot number											*/
/*				ipslot = IP card slot number 0 ~ 3 for VME, 0 ~ 5 for IOC			*/
/*				vectornum = interrupt vector number									*/
/*				doram = 0, output with register, = 1, output with RAM @ClockRate	*/
/*				clockSource = 0, internal, = 1, external							*/
/*				clockRate = 0 ~ 13, clock rate for output if selected. see manual	*/
/*				inhibit = 0, inhibit disabled, = 1, inhibit enabled					*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy8402_Configure(int vmeslot, char ipslot, int vectornum, int doram, 
							   int clockSource, int clockRate, int inhibit);
/*----------------------------------------------------------------------------------*/
/*Function:		Write One Channel													*/
/*----------------------------------------------------------------------------------*/
/*Description:	Output data to a DAC channel of 8402								*/
/*Prototype:	long Hy8402_Write(USHORT DAC_ch, USHORT value)						*/
/*Arguements:	ADC_ch = DAC output channel 0 ~ 15									*/
/*				value = unsigned 16bit value										*/
/*Return:		0 is successfull, negative please see error definition				*/
/*----------------------------------------------------------------------------------*/
	long Hy8402_Write(int DAC_ch, USHORT value);

/*----------------------------------------------------------------------------------*/
/*Function:		Write Waveform Data		 											*/
/*----------------------------------------------------------------------------------*/
/*Description:	Output the whole waveform data (64k) to a DAC channel memory area	*/
/*Prototype:	long Hy8402_Write(int DAC_ch, USHORT *value)						*/
/*Arguements:	ADC_ch = DAC output channel 0 ~ 15									*/
/*				*value = pointer to a 32k word array								*/
/*Return:		0 is successfull, negative please see error definition				*/
/*----------------------------------------------------------------------------------*/
	long Hy8402_Write(int DAC_ch, USHORT *value);

};
#if PLATFORM == 1									//windows
extern "C" HYTECAPI_API CHy8402* CreateCHy8402();
extern "C" HYTECAPI_API void DeleteCHy8402( CHy8402* instance );
#else											//linux
extern "C" CHy8402* CreateCHy8402();
extern "C" void DeleteCHy8402( CHy8402* instance );
#endif

