/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770	*/
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566	*/
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/
/*                                                                              */
/*				IOC9010 PROJECT	--- Hytec IP Card API Functions 				*/
/*							for Windows and Linux Platforms	    				*/
/*                                IP Card Type: 85156                           */
/********************************************************************************/
/*  Source file name      :-      Hy85156.h										*/
/*  Definition header file:		  HYTECIPAPI.h									*/
/*  Platform			  :		  Linux, Windows XP, Windows 2000               */
/*  Initial creation date :-      11-AUG-2006                                   */
/*                                                                              */
/*  Original Developers   :-      Jim S Chen                                    */
/********************************************************************************/
/*                                                                              */
/*  Description :-                                                              */
/*                                                                              */
/*  This header file is designed for Hytec API function library of 8515 and		*/
/*	8516 IP card. It defines the class prototype, error code, masks and 		*/
/*	registers of 8515/8516.    													*/
/*																				*/
/*  The design of this interface is cross platforms, which means the code 		*/
/*	can be used in operating systems like Windows, Linux etc without changes.	*/
/*                                                                              */
/*  The language used is C++ but the final product -- the Dynamic Link library	*/
/*	(dll) or the shared library can be used by either C or C++ applications.	*/
/*																				*/
/*  (C)2006 Hytec Electronics Ltd.												*/
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Modification record :-                                                      */
/*                                                                              */
/*  Date modified |     Person    |   Description of modification | Version     */
/* ---------------|---------------|-------------------------------------------- */
/*    11/08/06    |	  Jim S Chen  |	  Initial copy				  |	0.0.1		*/
/*                |               |                                             */
/*                                                                              */
/********************************************************************************/
#include "HytecIPAPI.h"

#if PLATFORM == 1

#define HYTECAPI_EXPORTS 1
#ifdef HYTECAPI_EXPORTS
#define HYTECAPI_API __declspec(dllexport)
#else
#define HYTECAPI_API __declspec(dllimport)
#endif
#include "C:\Hytec\IOC9010\IOCGeneric\IOCGeneric\IOCGeneric.h"

#else

#include "IOCGeneric.h"

#endif


//error code definition
#define NOT_85156_IP -85156010
#define Hy85156_NOT_CONFIGURED -85156011
#define PORT_INVALID_85156 -85156012
#define BAUD_RATE_INVALID -85156013
#define PARITY_INVALID -85156014
#define DATA_BITS_INVALID -85156015
#define STOP_BITS_INVALID -85156016
#define FLOW_CONTROL_INVALID -85156017

//signature definition
#define SIGNATURE_8515 0x8515
#define SIGNATURE_8516 0x8516

//register address defines
#define INTERRUPT_REG_85156 0x00*2
#define INT_SOURCE1_85156 0x01*2
#define INT_SOURCE2_85156 0x02*2
#define INT_SOURCE3_85156 0x03*2
#define TIMER_CONTROL_85156 0x04*2
#define TIMER_CONTROL0_85156 0x05*2
#define TIMER_COUNTER_LS_85156 0x06*2
#define TIMER_COUNTER_MS_85156 0x07*2
#define MODE_REG_85156 0x08*2
#define SLEEP_REG_85156 0x0B*2
#define REVISION_REG_85156 0x0C*2
#define ID_REG_85156 0x0D*2
#define CSR_85156 0x0E*2

#define PORT_OFFSET 0x20

#define RHR_85156 0x00
#define THR_85156 RHR_85156
#define DLL_85156 RHR_85156
#define DLM_85156 0x01
#define IER_85156 DLM_85156
#define ISR_85156 0x02
#define FCR_85156 ISR_85156
#define LCR_85156 0x03
#define MCR_85156 0x04
#define LSR_85156 0x05
#define MSR_85156 0x06
#define SPR_85156 0x07
#define FCTR_85156 0x08
#define EFR_85156 0x09
#define TXCNT_85156 0x0A
#define TXTRG_85156 TXCNT_85156
#define RXCNT_85156 0x0B
#define RXTRG_85156 RXCNT_85156
#define RCHAR_85156 0x0C
#define XOFF1_85156 RCHAR_85156
#define XOFF2_85156 0x0D
#define XON1_85156 0x0E
#define XON2_85156 0x0F

//mask definition
#define PARITY_MASK_85156 0xC7
#define NO_PARITY_85156 0x00
#define ODD_PARITY_85156 0x08
#define EVEN_PARITY_85156 0x18

#define DATABIT_MASK_85156 0xFC
#define DATA_5_85156 0x00
#define DATA_6_85156 0x01
#define DATA_7_85156 0x02
#define DATA_8_85156 0x03

#define STOPBIT_MASK_85156 0xFB
#define STOPBIT_1_85156 0x00
#define STOPBIT_2_85156 0x04

#define ENABLE_DIVISOR_MASK_85156 0x80
#define ENABLE_FIFO_85156 0x01
#define THR_EMPTY_85156 0x20
#define ENABLE_RHR_INT_85156 0x01
#define ENABLE_THR_INT_85156 0x02
#define ENABLE_RCVLINE_INT_85156 0x04
#define ENABLE_MODEM_INT_85156 0x08
#define FCR_RXFRESET_85156 0x02
#define FCR_TXFRESET_85156 0x04

#define INTERNAL_BIT_85156				0x10

#define BAUDRATE_100_DLL_85156			0x00
#define BAUDRATE_100_DLM_85156			0x09

#define BAUDRATE_400_DLL_85156			0x00
#define BAUDRATE_400_DLM_85156			0x09

#define BAUDRATE_600_DLL_85156			0x80
#define BAUDRATE_600_DLM_85156			0x01

#define BAUDRATE_1200_DLL_85156			0xC0
#define BAUDRATE_1200_DLM_85156			0x00

#define BAUDRATE_2400_DLL_85156			0x80
#define BAUDRATE_2400_DLM_85156			0x00

#define BAUDRATE_4800_DLL_85156			0xC0
#define BAUDRATE_4800_DLM_85156			0x00

#define BAUDRATE_9600_DLL_85156			0x60
#define BAUDRATE_9600_DLM_85156			0x00

#define BAUDRATE_19200_DLL_85156		0x30
#define BAUDRATE_19200_DLM_85156		0x00

#define BAUDRATE_38400_DLL_85156		0x18
#define BAUDRATE_38400_DLM_85156		0x00

#define BAUDRATE_57600_DLL_85156		0x04
#define BAUDRATE_57600_DLM_85156		0x00

#define BAUDRATE_76800_DLL_85156		0x0C
#define BAUDRATE_76800_DLM_85156		0x00

#define BAUDRATE_115200_DLL_85156		0x02
#define BAUDRATE_115200_DLM_85156		0x00

#define BAUDRATE_153600_DLL_85156		0x06
#define BAUDRATE_153600_DLM_85156		0x00

#define BAUDRATE_230400_DLL_85156		0x04
#define BAUDRATE_230400_DLM_85156		0x00

#define BAUDRATE_460800_DLL_85156		0x02
#define BAUDRATE_460800_DLM_85156		0x00

#define BAUDRATE_921600_DLL_85156		0x01
#define BAUDRATE_921600_DLM_85156		0x00

#define USE_HARDWARE_FLOW_TEST_85156	0x10
#define CTS_DTR_TEST_85156			0x20
#define RTS_DTR_CTS_DSR_ENABLE_85156		0xC0
#define DTR_DSR_SELECT_85156		0x04
#define USE_SOFTWARE_FLOW_TEST_85156	0x0F
#define SOFTWARE_FLOW_MASK_85156	0x0F

#define FLUSH_TX_FIFO 0x04
#define FLUSH_RX_FIFO 0x02
#define COM_0_TX_FLUSH 0x0001
#define COM_0_RX_FLUSH 0x0002
#define COM_1_TX_FLUSH 0x0004
#define COM_1_RX_FLUSH 0x0008
#define COM_2_TX_FLUSH 0x0010
#define COM_2_RX_FLUSH 0x0020
#define COM_3_TX_FLUSH 0x0040
#define COM_3_RX_FLUSH 0x0080
#define COM_4_TX_FLUSH 0x0100
#define COM_4_RX_FLUSH 0x0200
#define COM_5_TX_FLUSH 0x0400
#define COM_5_RX_FLUSH 0x0800
#define COM_6_TX_FLUSH 0x1000
#define COM_6_RX_FLUSH 0x2000
#define COM_7_TX_FLUSH 0x4000
#define COM_7_RX_FLUSH 0x8000

#define FIFO_SIZE 64
#define FIFO_SIZE 64
#define MAX_PORT_NO 7


// This class is exported from the HYTECAPI.dll
#if PLATFORM == 1									//windows
class HYTECAPI_API CHy85156 : public CIOCGeneric
#else											//linux
class CHy85156 : public CIOCGeneric
#endif
{
private:
	long m_ErrorCode;							// global error code holder
	int initIPSlot;								// remember the IP slot when configures
	int initVMESlot;

public:
	CHy85156(void);
	CHy85156(int vmeslot, char ipslot);
	~CHy85156(void);

	long GetErrorCode()
	{
		return m_ErrorCode;
	}
	
/*----------------------------------------------------------------------------------*/
/*Function:		Configure 85156 IP card												*/
/*----------------------------------------------------------------------------------*/
/*Description:	85156 IP configuration routine										*/
/*				according to the settings.											*/
/*Prototype:	long Hy85156_Configure(int vmeslot, char ipslot, char vector,		*/
/*				USHORT delay, int halfduplex, int delay485)							*/
/*Arguements:	vmeslot = The number of the VME carrier card on which the IP card	*/
/*					resides. The carrier must be initialised before this routine.	*/
/*					This param is only for VME.										*/
/*																					*/
/*				ipslot = IP card slot number 0 ~ 3 for VME, 0 ~ 5 for IOC			*/
/*																					*/
/*				vector = The interrupt vector number [1..255] designated to this IP	*/
/*					card. The card will not work without interrupts. Different		*/
/*					physical 8515/8516 cards may share the same interrupt number.	*/
/*					segment of the interrupt vector. only the upper 8 bits			*/
/*																					*/
/*				delay = For Positive Values -										*/
/*					Determine the number of clock ticks between timer interrupts.	*/
/*					The clock frequency is 125 KHz. The driver checks all active	*/
/*					channels of the serial card for characters that need to be read */	
/*					or written at every interrupt delay.							*/
/*																					*/
/*					For Negative Values -											*/
/*					The modulus of the Value sets the depth at which the RX FIFO	*/
/*					is filled and/or the TX FIFO is emptied before the serail IC	*/
/*					generates an Interrupt. Once an interrupt is detected, the		*/
/*					driver checks all active channels of the serial card for		*/
/*					characters that need to be read or written at every interrupt	*/
/*					delay.															*/
/*																					*/
/*					The modulus of intdelay should not be chosen to be too big for	*/
/*					a given data rate, keeping in mind that all channels have 64	*/
/*					byte FIFOs.														*/
/*																					*/
/*				halfduplex = delay the interrupt vector.							*/
/*					0:  Full duplex mode. Select this when using the 8515,			*/
/*						(for RS232) or full duplex RS485 mode when using on the		*/
/*						8516.														*/
/*					1:  Selects 'auto 485 mode' of the XR16L788 UART chip on the	*/
/*						8516. Do not select this option when using the 8515.		*/
/*																					*/
/*				delay485 = A number between 0 and 15. This parameter is only used if*/
/*					halfduplexmode = 1. It specifies the delay time in terms of		*/
/*					bits (depends on baudrate) after the last stop bit of the last	*/
/*					transmitted character before the state of the RTS# output is	*/
/*					changed to enable receiving.									*/
/*																					*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy85156_Configure(int vmeslot, char ipslot, char vector, int delay,
							int halfduplex, int delay485);

/*----------------------------------------------------------------------------------*/
/*Function:		Configure 85156 COM Port											*/
/*----------------------------------------------------------------------------------*/
/*Description:	set up one 85156 COM port											*/
/*Prototype:	Hy85156_ComsSetup(int port, USHORT baud, char parity,				*/
/*				int numstopbits, int numdatabits, char flowctr)						*/
/*Arguements:	port = serial channel 0 ~ 7											*/
/*				baud = baud rate divisor. This can be the follwoing number only		*/
/*					baud setting	value to register	internal loopback			*/
/*					------------------------------------------------------			*/
/*						100			2304 (0x0900)			1						*/
/*						400			2304 (0x0900)			0						*/
/*						600			 384 (0x0180)			1						*/
/*						1200		 192 (0x00C0)			1						*/
/*						2400		 128 (0x0080)			0						*/
/*						4800		 192 (0x00C0)			0						*/
/*						9600		  96 (0x0060)			0						*/
/*						19200		  48 (0x0030)			0						*/
/*						38400		  24 (0x0018)			0						*/
/*						57600		   4 (0x0004)			1						*/
/*						76800		  12 (0x000C)			0						*/
/*						115200		   2 (0x0002)			1						*/
/*						153600		   6 (0x0006)			0						*/
/*						230400		   4 (0x0004)			0						*/
/*						460800		   2 (0x0002)			0						*/
/*						921600		   1 (0x0001)			0						*/
/*				parity = parity setting												*/
/*					"N"	no parity													*/
/*					"O"	odd parity													*/
/*					"E"	even parity													*/
/*				numdatabits = data bits												*/
/*					=5, 6, 7, 8	for 5,6,7,8 bits									*/
/*				numstopbits = sstop bits											*/
/*					1	1 stop bit													*/
/*					4	1 1/2 stop bit if data bit = 5								*/
/*					4	2 stop bit if data bit = 6,7,8								*/
/*				flowctr = flow control byte											*/
/*					0 0 X X X X X X			FCR_RXFRESET_85156						*/
/*					[_]	| | [_____]													*/
/*					 |	| |	   |													*/
/*			   no used	| |	software flow control									*/
/*						| | 0 0 0 0	no software flow control						*/
/*						| | 1 1 1 1	transmit Xon1 and Xon2 / Xoff1 and Xoff2		*/
/*						| |			and receiver compares  Xon1 and Xon2 / Xoff1	*/
/*						| |			and Xoff2										*/
/*						| =0 disable hardware flow control, =1 enable				*/
/*					    =0 use RTS/CTS, =1 use DTR/DSR pair							*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy85156_ComsSetup(int port, ULONG baud, char parity, int numstopbits, int numdatabits,
							char flowctr);

/*----------------------------------------------------------------------------------*/
/*Function:		Send Data															*/
/*----------------------------------------------------------------------------------*/
/*Description:	Write data to specified serial channel								*/
/*Prototype:	long Hy85156_Write(int port, UCHAR *value, int length)				*/
/*Arguements:	port = serial channel 0 ~ 7											*/
/*				*value = pointer to the 8 bit value array							*/
/*				length = length of the 8 bit value array							*/
/*Return:		number of bytes written												*/
/*				if error, negative code in m_errorcode								*/
/*----------------------------------------------------------------------------------*/
	long Hy85156_Write(int port, UCHAR *value, int length);

/*----------------------------------------------------------------------------------*/
/*Function:		Read receving values												*/
/*----------------------------------------------------------------------------------*/
/*Description:	Read from receiving buffer of one channel in the 8515 IP card		*/
/*Prototype:	long Hy85156_Read(int port, UCHAR *value)							*/
/*Arguements:	port = serial channel 0 ~ 7											*/
/*				*value = pointer to the 8 bit value array							*/
/*Return:		number of bytes read												*/
/*				if error,  negative code in m_errorcode								*/
/*----------------------------------------------------------------------------------*/
	long Hy85156_Read(int port, UCHAR *value);

/*----------------------------------------------------------------------------------*/
/*Function:		Flush the serial buffer												*/
/*----------------------------------------------------------------------------------*/
/*Description:	Flush a serial buffer of the specified channel						*/
/*Prototype:	long Hy85156_Flush(int port, bool TX, bool RX)						*/
/*Arguements:	port = serial channel 0 ~ 7											*/
/*				TX = true to flush the sending buffer, false not					*/
/*				RX = true to flush the receiving buffer, false not					*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy85156_Flush(int port, bool TX, bool RX);
	
/*----------------------------------------------------------------------------------*/
/*Function:		Flush multi-serial buffers											*/
/*----------------------------------------------------------------------------------*/
/*Description:	Flush serial buffers according to the mask							*/
/*Prototype:	long Hy85156_Flush(USHORT mask)										*/
/*Arguements:	mask = serial channel mask											*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy85156_Flush(USHORT mask);

};

#if PLATFORM == 1									//windows
extern "C" HYTECAPI_API CHy85156* CreateCHy85156();
extern "C" HYTECAPI_API void DeleteCHy85156( CHy85156* instance );
#else											//linux
extern "C" CHy85156* CreateCHy85156();
extern "C" void DeleteCHy85156( CHy85156* instance );
#endif

