/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770	*/
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566	*/
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/
/*                                                                              */
/*				IOC9010 PROJECT	--- Hytec IP Card API Functions 				*/
/*							for Windows and Linux Platforms	    				*/
/*                                IP Card Type: 8601                            */
/********************************************************************************/
/*  Source file name      :-      Hy8601.h										*/
/*  Definition header file:		  HYTECIPAPI.h									*/
/*  Platform			  :		  Linux, Windows XP, Windows 2000               */
/*  Initial creation date :-      11-AUG-2006                                   */
/*                                                                              */
/*  Original Developers   :-      Jim S Chen                                    */
/********************************************************************************/
/*                                                                              */
/*  Description :-                                                              */
/*                                                                              */
/*  This header file is designed for Hytec API function library of 8601 IP card.*/
/*  It defines the class prototype, error code, masks and registers of 8601.    */
/*																				*/
/*  The design of this interface is cross platforms, which means the code 		*/
/*	can be used in operating systems like Windows, Linux etc without changes.	*/
/*                                                                              */
/*  The language used is C++ but the final product -- the Dynamic Link library	*/
/*	(dll) or the shared library can be used by either C or C++ applications.	*/
/*																				*/
/*  (C)2006 Hytec Electronics Ltd.												*/
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Modification record :-                                                      */
/*                                                                              */
/*  Date modified |     Person    |   Description of modification | Version     */
/* ---------------|---------------|-------------------------------------------- */
/*    11/08/06    |	  Jim S Chen  |	  Initial copy				  |	0.0.1		*/
/*                |               |                                             */
/*                                                                              */
/********************************************************************************/
#include "HytecIPAPI.h"

#if PLATFORM == 1

#define HYTECAPI_EXPORTS 1
#ifdef HYTECAPI_EXPORTS
#define HYTECAPI_API __declspec(dllexport)
#else
#define HYTECAPI_API __declspec(dllimport)
#endif
#include "C:\Hytec\IOC9010\IOCGeneric\IOCGeneric\IOCGeneric.h"

#else

#include "IOCGeneric.h"

#endif



//error code definition
#define NOT_8601_IP -8601010
#define Hy8601_NOT_CONFIGURED -8601011
#define AXIS_INVALID -8601012
#define STARTSPEED_INVALID -8601013
#define MAXSPEED_INVALID -8601014
#define RAMP_INVALID -8601015

//mask definition
#define MOTOR_RESET 0x0001
#define MOTOR_NEGLIMIT 0x0002
#define MOTOR_POSILIMIT 0x0004
#define MOTOR_HOMELIMIT 0x0008
#define MOTOR_DRIVEFAULT 0x0010
#define MOTOR_GO 0x0020
#define MOTOR_JOG 0x0040
#define MOTOR_ENCODER_DETECTED 0x0080
#define MOTOR_USE_ENCODER 0x0100
#define NOT_USED1 0x0200
#define NOT_USED2 0x0400
#define MOTOR_DIRECTION 0x0800
#define MOTOR_ABORT 0x1000
#define MOTOR_DONE 0x2000
#define MOTOR_INTERRUPT 0x4000
#define MOTOR_STOP_AT_HOME 0x8000

//signature definition
#define SIGNATURE_8601 0x8601

//register address defines
#define CSR_8601 0x000E
#define STEP_COUNTER_LOW_8601 0x0000
#define STEP_COUNTER_HIGH_8601 0x0002
#define ABSOLUTE_POSITION_LOW_8601 0x0004
#define ABSOLUTE_POSITION_HIGH_8601 0x0006
#define ATART_STOP_8601 0x0008
#define MAX_SPEED_8601 0x000A
#define RAMP_8601 0x000C

// This class is exported from the HYTECIPS.dll
#if PLATFORM == 1									//windows
class HYTECAPI_API CHy8601 : public CIOCGeneric
#else											//linux
class CHy8601 : public CIOCGeneric
#endif
{
private:
	long m_ErrorCode;						// global error code holder
	int initIPSlot;								// remember the IP slot when configures
	int initVMESlot;

public:
	CHy8601(void);
	CHy8601(int vmeslot, char ipslot);
	~CHy8601(void);

	long GetErrorCode()
	{
		return m_ErrorCode;
	}
	
/*----------------------------------------------------------------------------------*/
/*Function:		Configure 8601 IP card												*/
/*----------------------------------------------------------------------------------*/
/*Description:	8601 IP configuration routine										*/
/*				according to the settings.											*/
/*Prototype:	long Hy8601_Configure(int vmeslot, char ipslot)						*/
/*Arguements:	vmeslot = VME slot number											*/
/*				ipslot = IP card slot number 0 ~ 3 for VME, 0 ~ 5 for IOC			*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy8601_Configure(int vmeslot, char ipslot);

/*----------------------------------------------------------------------------------*/
/*Function:		Setup one 8601 motor channel										*/
/*----------------------------------------------------------------------------------*/
/*Description:	8601 IP motor channel setup routine									*/
/*				according to the specified channel settings.						*/
/*Prototype:	long Hy8601_MotorSetup(int axis, int start, int max, int ramp)		*/
/*Arguements:	axis = motor channel (0 ~ 3)										*/
/*				start = start speed (0 ~ 65535)							 			*/
/*				max = max speed (0 ~ 65535)								 			*/
/*				ramp = ramp setting (64 ~ 65535 with granularity of 64)	 			*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy8601_MotorSetup(int axis, int start, int max, int ramp);

/*----------------------------------------------------------------------------------*/
/*Function:		Move Steps for one motor channel									*/
/*----------------------------------------------------------------------------------*/
/*Description:	This routine sets the target steps to move and directions			*/
/*				then initilise the movement.										*/
/*Prototype:	long Hy8601_Move(int axis, long step)								*/
/*Arguements:	axis = motor channel (0 ~ 3)										*/
/*				step = step to go. positive value goes towards the positive limit	*/
/*					switch,	negative goes towards the negative limit switch			*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy8601_Move(int axis, long step);

/*----------------------------------------------------------------------------------*/
/*Function:		Read encoder for one motor channel									*/
/*----------------------------------------------------------------------------------*/
/*Description:	This routine reads the absolute position of the motor				*/
/*Prototype:	long Hy8601_ReadEncoder(int axis, unsigned long *presentpos)		*/
/*Arguements:	axis = motor channel (0 ~ 3)										*/
/*				*presentpos = pointer to the variable that holds the absolute		*/
/*					position of the specified motor channel							*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy8601_ReadEncoder(int axis, unsigned long int *presentpos);
	
/*----------------------------------------------------------------------------------*/
/*Function:		Read status word of one motor channel								*/
/*----------------------------------------------------------------------------------*/
/*Description:	This routine reads the status word of the specified motor channel	*/
/*Prototype:	long Hy8601_Status(int axis, USHORT mask, USHORT *status)			*/
/*Arguements:	axis = motor channel (0 ~ 3)										*/
/*				mask = bit mask for the needed status								*/
/*				*status = pointer to the variable that holds the status				*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy8601_Status(int axis, USHORT mask, USHORT *status);

/*----------------------------------------------------------------------------------*/
/*Function:		Test Done bit														*/
/*----------------------------------------------------------------------------------*/
/*Description:	This routine tests the done bit in the status of the specified		*/
/*				motor channel														*/
/*Prototype:	bool Hy8601_Done(int axis)											*/
/*Arguements:	axis = motor channel (0 ~ 3)										*/
/*Return:		true if the done bit set to 1, false if the done bite set to 0		*/
/*				if error, error in m_ErrorCode										*/
/*----------------------------------------------------------------------------------*/
	bool Hy8601_Done(int axis);

/*----------------------------------------------------------------------------------*/
/*Function:		Test Driver Fault													*/
/*----------------------------------------------------------------------------------*/
/*Description:	This routine tests if the driver is faulty or not of the specified	*/
/*				motor channel														*/
/*Prototype:	bool Hy8601_DriverFault(int axis)									*/
/*Arguements:	axis = motor channel (0 ~ 3)										*/
/*Return:		true if faulty, false if the motor driver is not faulty				*/
/*				if error, error in m_ErrorCode										*/
/*----------------------------------------------------------------------------------*/
	bool Hy8601_DriverFault(int axis);

/*----------------------------------------------------------------------------------*/
/*Function:		Detect Encoder														*/
/*----------------------------------------------------------------------------------*/
/*Description:	This routine detects if the specified motor has an encoder			*/
/*Prototype:	bool Hy8601_EncoderDetected(int axis)								*/
/*Arguements:	axis = motor channel (0 ~ 3)										*/
/*Return:		true if has encoder, false don't have								*/
/*				if error, error in m_ErrorCode										*/
/*----------------------------------------------------------------------------------*/
	bool Hy8601_EncoderDetected(int axis);

/*----------------------------------------------------------------------------------*/
/*Function:		Test home limit 													*/
/*----------------------------------------------------------------------------------*/
/*Description:	This routine tests if the specified motor has reached its home		*/
/*				limit switch														*/
/*Prototype:	bool Hy8601_HomeLimit(int axis)										*/
/*Arguements:	axis = motor channel (0 ~ 3)										*/
/*Return:		true if has reached, false if not									*/
/*				if error, error in m_ErrorCode										*/
/*----------------------------------------------------------------------------------*/
	bool Hy8601_HomeLimit(int axis);

/*----------------------------------------------------------------------------------*/
/*Function:		Test positive limit 												*/
/*----------------------------------------------------------------------------------*/
/*Description:	This routine tests if the specified motor has reached its positive	*/
/*				limit switch														*/
/*Prototype:	bool Hy8601_PosiLimit(int axis)										*/
/*Arguements:	axis = motor channel (0 ~ 3)										*/
/*Return:		true if has reached, false if not									*/
/*				if error, error in m_ErrorCode										*/
/*----------------------------------------------------------------------------------*/
	bool Hy8601_PosiLimit(int axis);

/*----------------------------------------------------------------------------------*/
/*Function:		Test negative limit 												*/
/*----------------------------------------------------------------------------------*/
/*Description:	This routine tests if the specified motor has reached its negative	*/
/*				limit switch														*/
/*Prototype:	bool Hy8601_NegaLimit(int axis)										*/
/*Arguements:	axis = motor channel (0 ~ 3)										*/
/*Return:		true if has reached, false if not									*/
/*				if error, error in m_ErrorCode										*/
/*----------------------------------------------------------------------------------*/
	bool Hy8601_NegaLimit(int axis);

/*----------------------------------------------------------------------------------*/
/*Function:		Set the CSR register 												*/
/*----------------------------------------------------------------------------------*/
/*Description:	This routine sets the specified motor channel CSR register			*/
/*Prototype:	long Hy8601_Settings(int axis, USHORT mask, USHORT value)			*/
/*Arguements:	axis = motor channel (0 ~ 3)										*/
/*				mask = bit mask														*/
/*				value = value to set												*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy8601_Settings(int axis, USHORT mask, USHORT value);

/*----------------------------------------------------------------------------------*/
/*Function:		Abort action		 												*/
/*----------------------------------------------------------------------------------*/
/*Description:	This routine sets the specified motor channel abort bit in the		*/
/*				CSR register														*/
/*Prototype:	long Hy8601_Abort(int axis, bool value)								*/
/*Arguements:	axis = motor channel (0 ~ 3)										*/
/*				value = true sets, false clears										*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy8601_Abort(int axis, bool value);

/*----------------------------------------------------------------------------------*/
/*Function:		Stop motor at home	 												*/
/*----------------------------------------------------------------------------------*/
/*Description:	This routine stops the specified motor channel to its home limit	*/
/*				switch																*/
/*Prototype:	long Hy8601_StopAtHome(int axis, bool value)						*/
/*Arguements:	axis = motor channel (0 ~ 3)										*/
/*				value = true sets, false clears										*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy8601_StopAtHome(int axis, bool value);

/*----------------------------------------------------------------------------------*/
/*Function:		Set use encoder		 												*/
/*----------------------------------------------------------------------------------*/
/*Description:	This routine sets either use the encoder of the specified motor   	*/
/*				or not																*/
/*Prototype:	long Hy8601_UseEncoder(int axis, bool value)						*/
/*Arguements:	axis = motor channel (0 ~ 3)										*/
/*				value = true sets, false clears										*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy8601_UseEncoder(int axis, bool value);

/*----------------------------------------------------------------------------------*/
/*Function:		Reset Motor			 												*/
/*----------------------------------------------------------------------------------*/
/*Description:	This routine resets the specified motor							 	*/
/*Prototype:	long Hy8601_Reset(int axis, bool value)								*/
/*Arguements:	axis = motor channel (0 ~ 3)										*/
/*				value = true resets, false do not reset								*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy8601_Reset(int axis, bool value);

/*----------------------------------------------------------------------------------*/
/*Function:		Use Interrupt		 												*/
/*----------------------------------------------------------------------------------*/
/*Description:	This routine sets either use the interrupt for specified motor		*/
/*				channel or not														*/
/*Prototype:	long Hy8601_Interrupt(int axis, bool value)							*/
/*Arguements:	axis = motor channel (0 ~ 3)										*/
/*				value = true uses, false do not use									*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy8601_Interrupt(int axis, bool value);

};

#if PLATFORM == 1									//windows
extern "C" HYTECAPI_API CHy8601* CreateCHy8601();
extern "C" HYTECAPI_API void DeleteCHy8601( CHy8601* instance );
#else											//linux
extern "C" CHy8601* CreateCHy8601();
extern "C" void DeleteCHy8601( CHy8601* instance );
#endif

