
#include <ctype.h>              /* for toupper() */
#include <stdio.h>
#include "vdb_structure.h"      /* for vista_chix */
#include "vdb_convert.h"        /* for conversion_record */
#include "vdb_descrip.h"        /* for vdb_descrip_s */
#include <vsys_routines.h>
#include <vdb_routines.h>
#include <vdb_errors.h>
#include <vdb_class.h>
#include <math.h>

#define ADC_A_ZERO_CAL  16  /* ADC A - Zero Raw Calibration */
#define ADC_A_POS_CAL   17  /* ADC A - Pos  Raw Calibration */
#define ADC_A_NEG_CAL   18  /* ADC A - Neg  Raw Calibration */
#define ADC_B_ZERO_CAL  19  /* ADC B - Zero Raw Calibration */
#define ADC_B_POS_CAL   20  /* ADC B - Pos  Raw Calibration */
#define ADC_B_NEG_CAL   21  /* ADC B - Neg  Raw Calibration */

#define TYPE_K 1
#define TYPE_T 2
#define TYPE_J 3

int main (int argc, char *argv) {

}


int pt100_conversion (
    vista_chix  *Chix,           /* Vista chix for channel */
    int         *numparams,     /* number of cparams passed */
    float       *Params,        /* pointer to array of cparams */
    int         *numvals,       /* number of values passed */
    float       *invalue,     /* pointer to array of unconverted values */
    float       *outvalue)     /* pointer to array of converted values */
{
  int     i = 0, x = 0, channelnum = 0;
  struct vdb_descrip_v desc;
  VDB_DATA_ITEM_LIST dlist[40]; 
  char       chan_name[12000];
  int *ptr;
  float refdata =0;
  short            iChannelCount;
  unsigned long    ADC_Value;
  unsigned int            refvalueA;
  unsigned int            refvalueB;
  double           VoltageA;
  double           VoltageB;
  double           Temperature;
  double           Rref = 100.0; /* Reference/Vish Resistor is 100R */
  double           IrefA;
  double           IrefB;
  double           A = 0.00390802;
  double           B = -0.000000580195;
  double			Gain = 0;

  ptr = (unsigned int *)Params;
  refvalueA = ptr[1];
  refvalueB = ptr[1];
  Gain = (double)Params[2];

  /*Get this channel name, cparam1 and 2 (the stored reference values),
   and cparam3 (the gain) */
  dlist[0].chix = *Chix; 
  dlist[0].event_class_number= VCLS_N_INFO; 
  dlist[0].event_field_mask= VFLD_M_CHNAME; 
  dlist[0].index= 0; 
  dlist[0].dtype= VDB_DTYPE_STRING; 
  dlist[0].size= 255; 
  dlist[0].data_addr= chan_name; 
  
  /*  dlist[1].chix = *Chix; 
  dlist[1].event_class_number = VCLS_N_CONVERSION; 
  dlist[1].event_field_mask = VFLD_M_CPARAM1; 
  dlist[1].dtype = VDB_DTYPE_UINT; 
  dlist[1].size = sizeof(unsigned int); 
  dlist[1].data_addr = (float *)&refvalueA; 

  dlist[2].chix = *Chix; 
  dlist[2].event_class_number = VCLS_N_CONVERSION; 
  dlist[2].event_field_mask = VFLD_M_CPARAM2; 
  dlist[2].dtype = VDB_DTYPE_FLOAT; 
  dlist[2].size = sizeof(float); 
  dlist[2].data_addr = &refvalueB;

  dlist[3].chix = *Chix; 
  dlist[3].event_class_number = VCLS_N_CONVERSION; 
  dlist[3].event_field_mask = VFLD_M_CPARAM3; 
  dlist[3].dtype = VDB_DTYPE_FLOAT; 
  dlist[3].size = sizeof(float); 
  dlist[3].data_addr = &Gain;
  */
  dlist[1].chix = *Chix; 
  dlist[1].event_class_number = VCLS_N_HARDWARE; 
  dlist[1].event_field_mask = VFLD_M_HPARAM5; 
  dlist[1].dtype = VDB_DTYPE_INT; 
  dlist[1].size = sizeof(int); 
  dlist[1].data_addr = &channelnum;

  vdb_zero_chix(&(dlist[2].chix)); 
  
  vdb_get_item (dlist); 

  //  vsys_msg_printf(0, "CONV:channel %s with value %f, refvalue %f, gain %f\n", chan_name, *invalue, refdata, Gain);
  
  ADC_Value = (long int)refvalueA;

  /* Calculate ADC A's Common Current */

  VoltageA = (double)(2.5 * ADC_Value) / (double)(16777216);
  IrefA = VoltageA/Rref;
  

  ADC_Value = (long int)refvalueB;
  
  /* Calculate ADC B's Common Current */
  VoltageB = (double)(2.5 * ADC_Value) / (double)(16777216);
  IrefB = VoltageB/Rref;
  
  ADC_Value = (long int)*invalue;
  
  if (channelnum < 7) {
    Temperature = (Rref - (VoltageA/IrefA)) / (-0.5 * (A*Rref + sqrt( (A*A * Rref*Rref) - 4*B*Rref * (Rref - VoltageA/IrefA)))); 
    /*    printf("Channel %2d- 0x%08x - %8d - %1.6f V - %1.1f°C\n",channelnum,
	 (unsigned int)ADC_Value,
	 (unsigned int)ADC_Value,
	 VoltageA/4,
	 Temperature); */
  } else {
    Temperature = (Rref - VoltageB/IrefB)/(-0.5 * (A*Rref + sqrt((A*A * Rref*Rref) - 4*B*Rref * (Rref - VoltageB/IrefB))));
  
    /*  printf("Channel %2d- 0x%08x - %8d - %1.6f V - %1.1f°C\n",channelnum,
	 (unsigned int)ADC_Value,
	 (unsigned int)ADC_Value,
	 VoltageB/4,
	 Temperature); */
  }
  *outvalue = Temperature;
  return (1); /* 1 == success */
}

int tc_conversion (
    vista_chix  *Chix,           /* Vista chix for channel */
    int         *numparams,     /* number of cparams passed */
    float       *Params,        /* pointer to array of cparams */
    int         *numvals,       /* number of values passed */
    float       *invalue,     /* pointer to array of unconverted values */
    float       *outvalue)     /* pointer to array of converted values */
{
  int     i = 0, x = 0, channelnum = 0;
  struct vdb_descrip_v desc;
  VDB_DATA_ITEM_LIST dlist[40]; 
  char       chan_name[12000];
 
  float refdata =0;
  short           iChannelCount;
  unsigned long   ADC_Value;
  int Thermocouple_Type;
  unsigned int *ptr;
	/* PT-100 / Cold Junction Temperature Variables */
    double			Vpt100;		/* PT100 Voltage */
    double			Ipt100;		/* PT100 Current */
    double			Rpt100;		/* PT100 Resistance */
    double			Tpt100;		/* PT100 Temperature */
    double           Rref = 100.0; /* Reference/Vish Resistor is 100R */
    double           A = 0.00390802;
    double           B = -0.000000580195;
    double			Gain = 0;
    unsigned int           refvalueA;
    double           Voltage;
    double           Temperature;
    double           TV; 
    double d0,d1,d2,d3,d4,d5,d6,d7,d8,d9,d10,d11,d12,d13;
    double a0,a1,a2;
    
    ptr = (unsigned int *)Params;
    refvalueA = ptr[1];
    //    refvalueB = ptr[1];
    Gain = (double)Params[2];

    /*Get this channel name, cparam2 (the stored reference value),
      cparam3 (the gain), hparam5 (the channel number),
      and hparam6 (the thermocouple type) */
  dlist[0].chix = *Chix; 
  dlist[0].event_class_number= VCLS_N_INFO; 
  dlist[0].event_field_mask= VFLD_M_CHNAME; 
  dlist[0].index= 0; 
  dlist[0].dtype= VDB_DTYPE_STRING; 
  dlist[0].size= 255; 
  dlist[0].data_addr= chan_name; 
  
  /*  dlist[1].chix = *Chix; 
  dlist[1].event_class_number = VCLS_N_CONVERSION; 
  dlist[1].event_field_mask = VFLD_M_CPARAM2; 
  dlist[1].dtype = VDB_DTYPE_UINT; 
  dlist[1].size = sizeof(unsigned int); 
  dlist[1].data_addr = (unsigned int *)&refvalueA; 

  dlist[2].chix = *Chix; 
  dlist[2].event_class_number = VCLS_N_CONVERSION; 
  dlist[2].event_field_mask = VFLD_M_CPARAM3; 
  dlist[2].dtype = VDB_DTYPE_DOUBLE; 
  dlist[2].size = sizeof(double); 
  dlist[2].data_addr = (double*)&Gain;
  */
  dlist[1].chix = *Chix; 
  dlist[1].event_class_number = VCLS_N_HARDWARE; 
  dlist[1].event_field_mask = VFLD_M_HPARAM5; 
  dlist[1].dtype = VDB_DTYPE_INT; 
  dlist[1].size = sizeof(int); 
  dlist[1].data_addr = (int *)&channelnum;

  dlist[2].chix = *Chix; 
  dlist[2].event_class_number = VCLS_N_HARDWARE; 
  dlist[2].event_field_mask = VFLD_M_HPARAM7; 
  dlist[2].dtype = VDB_DTYPE_INT; 
  dlist[2].size = sizeof(int); 
  dlist[2].data_addr = &Thermocouple_Type;

  vdb_zero_chix(&(dlist[3].chix)); 
  
  vdb_get_item (dlist); 

  //  vsys_msg_printf(0, "CONV:channel %s, therm type %d with value %f, refvalue %d, gain %f\n", chan_name, Thermocouple_Type, *invalue, refvalueA, Gain);
  
  if (Thermocouple_Type < 0 || Thermocouple_Type > 3) {
    vsys_msg_printf(0, "Warning: Thermocouple type %d invalid for channel %s. Please check database. Defaulting to \"K\".", chan_name, Thermocouple_Type);
    Thermocouple_Type = TYPE_K;
  }

  ADC_Value = (long int)refvalueA;

  /* Convert PT100 Value into Volts */
  Vpt100 = ((double)(ADC_Value) - 0x800000)*(double)(2.5/Gain) / (double)(16777216);
  
  /* Calculate Current through PT100 */
  Ipt100 = (double)(2.5 - Vpt100)/(25000);
  
  /* Calculate Resistance of PT100 */
  Rpt100 = Vpt100/Ipt100;
  
  /* Calculate Temperature of PT100 */
  Tpt100 = (Rref - Rpt100)/(-0.5 * (A*Rref + sqrt((A*A * Rref*Rref) - 4*B*Rref * (Rref - Rpt100))));                               
  
  /*  printf("PT100 - 0x%08x - %8d - %1.6f V - %1.6f A - %1.6f R -  %1.2f°C\n",	(unsigned int)ADC_Value,
	 (unsigned int)ADC_Value,
	 Vpt100,
	 Ipt100,
	 Rpt100,
	 Tpt100); */
  
  switch (Thermocouple_Type)
    {
    case TYPE_K:
      /* Convert PT100 Temperature to an equivalent K Type Thermocouple Voltage */
      if(Tpt100 < 0.000) {
		  d0 =  3.94501280250E-02;
		  d1 =  2.36223735980E-05;
		  d2 = -3.28589067840E-07;
		  d3 = -4.99048287770E-09;
		  d4 = -6.75090591730E-11;
		  d5 = -5.74103274280E-13;
		  d6 = -3.10888728940E-15;
		  d7 = -1.04516093650E-17;
		  d8 = -1.98892668780E-20;
		  d9 = -1.63226974860E-23;
		  
		  TV =  d0 * (double)(pow(Tpt100,1))
		    + d1 * (double)(pow(Tpt100,1))
		    + d2 * (double)(pow(Tpt100,2))   
		    + d3 * (double)(pow(Tpt100,3))   
		    + d4 * (double)(pow(Tpt100,4))   
		    + d5 * (double)(pow(Tpt100,5))   
		    + d6 * (double)(pow(Tpt100,6))   
		    + d7 * (double)(pow(Tpt100,7))   
		    + d8 * (double)(pow(Tpt100,8))   
		    + d9 * (double)(pow(Tpt100,9));   
		}
		else {
		  d0 = -1.76004136860E-02;
		  d1 =  3.89212049750E-02;
		  d2 =  1.85587700320E-05;
		  d3 = -9.94575928740E-08;
		  d4 =  3.18409457190E-10;
		  d5 = -5.60728448890E-13;
		  d6 =  5.60750590590E-16;
		  d7 = -3.20207200030E-19;
		  d8 =  9.71511471520E-23;
		  d9 = -1.21047212750E-26;
		  a0 =  0.118597600000E00;
		  a1 = -0.118343200000E-03;
		  a2 =  0.126968600000E03;
		  
		  TV =  d0
		    + d1 * (double)(pow(Tpt100,1))
		    + d2 * (double)(pow(Tpt100,2))   
		    + d3 * (double)(pow(Tpt100,3))   
		    + d4 * (double)(pow(Tpt100,4))   
		    + d5 * (double)(pow(Tpt100,5))   
		    + d6 * (double)(pow(Tpt100,6))   
		    + d7 * (double)(pow(Tpt100,7))   
		    + d8 * (double)(pow(Tpt100,8))   
		    + d9 * (double)(pow(Tpt100,9));   
		  + ((double) a0 * exp(a1 * (Tpt100-a2) * (Tpt100-a2))); 
		}
		break;
		
    case TYPE_T:
      /* Convert PT100 Temperature to an equivalent T Type Thermocouple Voltage */
      if(Tpt100 < 0.000) {
	d0 =  0.387481063640E-01;
	d1 =  0.441944343470E-04;
	d2 =  0.118443231050E-06;
	d3 =  0.200329735540E-07;
	d4 =  0.901380195590E-09;
	d5 =  0.226511565930E-10;
	d6 =  0.360711542050E-12;
	d7 =  0.384939398830E-14;
	d8 =  0.282135219250E-16;
	d9 =  0.142515947790E-18;
	d10 =  0.487686622860E-21;
	d11 =  0.107955392700E-23;
	d12 =  0.139450270620E-26;
	d13 =  0.797951539270E-30;
	
	TV =  d0 * (double)(pow(Tpt100,1))
	  + d1 * (double)(pow(Tpt100,2))
	  + d2 * (double)(pow(Tpt100,3))   
	  + d3 * (double)(pow(Tpt100,4))   
	  + d4 * (double)(pow(Tpt100,5))   
	  + d5 * (double)(pow(Tpt100,6))   
	  + d6 * (double)(pow(Tpt100,7))   
	  + d7 * (double)(pow(Tpt100,8))   
	  + d8 * (double)(pow(Tpt100,9))   
	  + d9 * (double)(pow(Tpt100,10))   
	  + d10 * (double)(pow(Tpt100,11))   
	  + d11 * (double)(pow(Tpt100,12))   
	  + d12 * (double)(pow(Tpt100,13))   
	  + d13 * (double)(pow(Tpt100,14));   
      }
      else {
	d0 =  0.387481063640E-01;
	d1 =  0.332922278800E-04;
	d2 =  0.206182434040E-06;
	d3 = -0.218822568460E-08;
	d4 =  0.109968809280E-10;
	d5 = -0.308157587720E-13;
	d6 =  0.454791352900E-16;
	d7 = -0.275129016730E-19;
	
	TV =  d0 * (double)(pow(Tpt100,1))
	  + d1 * (double)(pow(Tpt100,2))
	  + d2 * (double)(pow(Tpt100,3))   
	  + d3 * (double)(pow(Tpt100,4))   
	  + d4 * (double)(pow(Tpt100,5))   
	  + d5 * (double)(pow(Tpt100,6))   
	  + d6 * (double)(pow(Tpt100,7))   
	  + d7 * (double)(pow(Tpt100,8));   
      }
      break;
      
    case TYPE_J:
      /* Convert PT100 Temperature to an equivalent J Type Thermocouple Voltage */
      if(Tpt100 < 760.000) {
	
	d0 =  0.503811878150E-01;
	d1 =  0.304758369300E-04;
	d2 = -0.856810657200E-07;
	d3 =  0.132281952950E-09;
	d4 = -0.170529583370E-12;
	d5 =  0.209480906970E-15;
	d6 = -0.125383953360E-18;
	d7 =  0.156317256970E-22;
	
	TV =  d0 * (double)(pow(Tpt100,1))
	  + d1 * (double)(pow(Tpt100,2))
	  + d2 * (double)(pow(Tpt100,3))   
	  + d3 * (double)(pow(Tpt100,4))   
	  + d4 * (double)(pow(Tpt100,5))   
	  + d5 * (double)(pow(Tpt100,6))   
	  + d6 * (double)(pow(Tpt100,7))   
	  + d7 * (double)(pow(Tpt100,8));   
      }
      else {
	d0 =  0.296456256810E03;
	d1 = -0.149761277860E01;
	d2 =  0.317871039240E-02;
	d3 = -0.318476867010E-05;
	d4 =  0.157208190040E-08;
	d5 = -0.306913690560E-12;
	
	TV =  d0
	  + d1 * (double)(pow(Tpt100,1))
	  + d2 * (double)(pow(Tpt100,2))   
	  + d3 * (double)(pow(Tpt100,3))   
	  + d4 * (double)(pow(Tpt100,4))   
	  + d5 * (double)(pow(Tpt100,5));
      }
      break;
    }
  
  /* Display ADC A */ 
  
  ADC_Value = (long int)*invalue;
  
  /* Convert Thermocouple Raw ADC Value into Volts */
  Vpt100 = ((double)(ADC_Value) - 0x800000)*(double)(2.5/Gain) / (double)(16777216);
  
  Voltage = ((double)(TV / 1000) + (double)(Vpt100)) * (double)(1000);
  
  switch (Thermocouple_Type)
    {
    case TYPE_K:
      /* Convert Voltage to Temperature - Using K Type Formula */
      if(Voltage < 0.00) {
	d0 =  0.0000000E00;
	d1 =  2.5173462E01;
	d2 = -1.1662878E00;
	d3 = -1.0833638E00;
	d4 = -8.9773540E-01;
	d5 = -3.7342377E-01;
	d6 = -8.6632643E-02;
	d7 = -1.0450598E-02;
	d8 = -5.1920577E-04;
	d9 =  0.0000000E00;
      }
      else if(Voltage <= 20.644) {
	d0 =  0.000000E00;
	d1 =  2.508355E01;
	d2 =  7.860106E-02;
	d3 = -2.503131E-01;
	d4 =  8.315270E-02;
	d5 = -1.228034E-02;
	d6 =  9.804036E-04;
	d7 = -4.413030E-05;
	d8 =  1.057734E-06;
	d9 = -1.052755E-08;
      }
      else {
	d0 = -1.318058E02;
	d1 =  4.830222E01;
	d2 = -1.646031E00;
	d3 =  5.464731E-02;
	d4 = -9.650715E-04;
	d5 =  8.802193E-06;
	d6 = -3.110810E-08;
	d7 =  0.000000E00;
	d8 =  0.000000E00;
	d9 =  0.000000E00;
      }
      
      Temperature = d0
	+ d1 * (double)(Voltage)
	+ d2 * (double)(pow(Voltage,2))   
	+ d3 * (double)(pow(Voltage,3))   
	+ d4 * (double)(pow(Voltage,4))   
	+ d5 * (double)(pow(Voltage,5))   
	+ d6 * (double)(pow(Voltage,6))   
	+ d7 * (double)(pow(Voltage,7))   
	+ d8 * (double)(pow(Voltage,8))   
	+ d9 * (double)(pow(Voltage,9));   
      break;
      
    case TYPE_T:
      /* Convert Voltage to Temperature - Using T Type Formula */
      if(Voltage < 0.00) {
	d0 =  0.0000000E00;
	d1 =  2.5949192E01;
	d2 = -2.1316967E-01;
	d3 =  7.9018692E-01;
	d4 =  4.2527777E-01;
	d5 =  1.3304473E-01;
	d6 =  2.0241446E-02;
	d7 =  1.2668171E-03;
      }
      else {
	d0 =  0.000000E00;
	d1 =  2.592800E01;
	d2 = -7.602961E-01;
	d3 =  4.637791E-02;
	d4 = -2.165394E-03;
	d5 =  6.048144E-05;
	d6 = -7.293422E-07;
	d7 =  0.000000E00;
      }
      
      Temperature = d0
	+ d1 * (double)(Voltage)
	+ d2 * (double)(pow(Voltage,2))   
	+ d3 * (double)(pow(Voltage,3))   
	+ d4 * (double)(pow(Voltage,4))   
	+ d5 * (double)(pow(Voltage,5))   
	+ d6 * (double)(pow(Voltage,6))   
	+ d7 * (double)(pow(Voltage,7));   
      break;
      
    case TYPE_J:
      /* Convert Voltage to Temperature - Using J Type Formula */
      if(Voltage < 0.00) {
	
	d0 =  0.0000000E00;
	d1 =  1.9528268E01;
	d2 = -1.2286185E00;
	d3 = -1.0752178E00;
	d4 = -5.9086933E-01;
	d5 = -1.7256713E-01;
	d6 = -2.8131513E-02;
	d7 = -2.3963370E-03;
	d8 = -8.3823321E-05;
	
      }
      else if(Voltage <= 42.919) {
	
	d0 =  0.000000E00; 
	d1 =  1.978425E01;
	d2 = -2.001204E-01;
	d3 =  1.036969E-02; 
	d4 = -2.549687E-04;
	d5 =  3.585153E-06; 
	d6 = -5.344285E-08;
	d7 =  5.099890E-10; 
	d8 =  0.000000E00; 
	
      }
      else {
	
	d0 = -3.11358187E03;
	d1 =  3.00543684E02;
	d2 = -9.94773230E00;
	d3 =  1.70276630E-01;
	d4 = -1.43033468E-03;
	d5 =  4.73886084E-06;
	d6 =  0.00000000E00;
	d7 =  0.00000000E00;
	d8 =  0.00000000E00;
	
      }
      Temperature = d0
	+ d1 * (double)(Voltage)
	+ d2 * (double)(pow(Voltage,2))   
	+ d3 * (double)(pow(Voltage,3))   
	+ d4 * (double)(pow(Voltage,4))   
	+ d5 * (double)(pow(Voltage,5))   
	+ d6 * (double)(pow(Voltage,6))   
	+ d7 * (double)(pow(Voltage,7))   
	+ d8 * (double)(pow(Voltage,8));   
      break;
    }		
  
  /*  printf("ADC A Channel %2d - Raw ADC - 0x%08x  Temp -  %1.1f°C\n",   (int)channelnum, 
	 (int)ADC_Value,
	 Temperature); */
  
  *outvalue = (float)Temperature;
  return (1); /* 1 == success */
}
