/******************************************************************************
*        ____
*      _-    -_    _------_
*     -        -_--        -_
*          _--------------_   -_
* 	_---              -_
*      _-                 ----___
*                         ---___
*      Copyright (c) 1999-2004 Vista Control Systems, Inc.
*      All rights reserved
*      176 Central Park Square * Los Alamos, N.M. 87544 * 505-662-2484
*
*      Filename: vhytec/io.c
*
**********************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include <ctype.h>
//#include <vsocket.h>
#include <vsys_time.h>
#include <vsys_internal.h>

#include "internal.h"
#include "slio.h"

int control_design = 0;
int showDecimal = 0;

/*
 * The description for a port
 */


#include "io.h"

#define SUCCESS VMB_OK
#define FAILURE (-1)

#define VMB_HYTEC_PORT 502

/*
 *********************************************************************
 */

static int stringError (vp_t *h, char *string) {
  (void) vsys_strncpy (h->errstr, string, sizeof h->errstr);
  return FAILURE;
}

/*
 *********************************************************************
 */

#define errINTERR(h) errInterr (h, __LINE__)

static int errInterr (vp_t *h, int line) {
  (void) sprintf (h->errstr, "VMBIO_INTERR, Internal error %d", line);
  return FAILURE;
}

/*
 *********************************************************************
 */

#define errINVARG(h) errInvarg (h, __LINE__)

static int errInvarg (vp_t *h, int line) {
  (void) sprintf (h->errstr, "VMBIO_INVARG, Invalid argument (%d)", line);
  return FAILURE;
}

/*
 *********************************************************************
 */

#define errPROTOCOL(h) errProtocol (h, __LINE__)

static int errProtocol (vp_t *h, int line) {
  (void) sprintf (h->errstr, "VMBIO_PROTOCOL, Protocol error %d", line);
  return FAILURE;
}

/*
 *********************************************************************
 */

#define errNOMEM(h) errNomem (h)

static int errNomem (vp_t *h) {
  return stringError (h, "VMBIO_NOMEM, Insufficient virtual memory");
}

/*
 *********************************************************************
 */

static int socketError (vp_t *h) {
  (void) stringError (h, vsocketStrerror (h->tcp.socklib));
  if (h->tcp.socket != NULL) {
    (void) vsocketClose (h->tcp.socket);
    h->tcp.socket = NULL;
    h->connected = 0;
  }
  return FAILURE;
}

/*
 *********************************************************************
 */

static int slError (vp_t *h) {
  return stringError (h, slStrerror (h->sl));
}

/*
 *********************************************************************
 */

static void hexdump (char* title, void* ptr, unsigned length) {

  unsigned char* c = ptr;
  unsigned i;
  FILE* file = stdout;
  vtime bintim;
  char asctim[32];

  (void) vsys_get_time (&bintim);
  (void) vsysc_asctime (asctim, sizeof asctim, &bintim);
  (void) fprintf (file, "DUMP %s (%u bytes) ptr=%lx time=%s",
		  title, length, (unsigned long) ptr, asctim);
  for (i = 0; i < length; i += 16) {
    unsigned count = length - i > 16 ? 16 : length - i;
    unsigned j;
/*     (void) fprintf (file, "\n%04X ", i); */
    (void) fprintf (file, showDecimal ? "\n%d" : "\n%04X",
		       i);
    for (j = 0; j < count; j++)
/*       (void) fprintf (file, " %02X", c[i + j]); */
      (void) fprintf (file, showDecimal ? " %d" : " %02X",
		      c[i + j]);
    if (count != 16)
      for (j = count; j < 16; j++)
	(void) fprintf (file, "   ");
    (void) fprintf (file, "  ");
    for (j = 0; j < count; j++)
      (void) fprintf (file, "%c", isprint (c[i+ j]) ? c[i + j] : '.');
  }
  (void) fprintf (file, "\n");
  (void) fflush (file);
}

/*
 *********************************************************************
 * Given a pointer to a buffer, calculate a CRC-16 for it and return it
 *********************************************************************
 */

static unsigned short int vmbcrc16 (unsigned char *bytp, int count) {

  /* CRC table for polynomial A001, generated by crctable utility */

  static unsigned short crcTable[256] = {
    0x0000, 0xC0C1, 0xC181, 0x0140, 0xC301, 0x03C0, 0x0280, 0xC241,
    0xC601, 0x06C0, 0x0780, 0xC741, 0x0500, 0xC5C1, 0xC481, 0x0440,
    0xCC01, 0x0CC0, 0x0D80, 0xCD41, 0x0F00, 0xCFC1, 0xCE81, 0x0E40,
    0x0A00, 0xCAC1, 0xCB81, 0x0B40, 0xC901, 0x09C0, 0x0880, 0xC841,
    0xD801, 0x18C0, 0x1980, 0xD941, 0x1B00, 0xDBC1, 0xDA81, 0x1A40,
    0x1E00, 0xDEC1, 0xDF81, 0x1F40, 0xDD01, 0x1DC0, 0x1C80, 0xDC41,
    0x1400, 0xD4C1, 0xD581, 0x1540, 0xD701, 0x17C0, 0x1680, 0xD641,
    0xD201, 0x12C0, 0x1380, 0xD341, 0x1100, 0xD1C1, 0xD081, 0x1040,
    0xF001, 0x30C0, 0x3180, 0xF141, 0x3300, 0xF3C1, 0xF281, 0x3240,
    0x3600, 0xF6C1, 0xF781, 0x3740, 0xF501, 0x35C0, 0x3480, 0xF441,
    0x3C00, 0xFCC1, 0xFD81, 0x3D40, 0xFF01, 0x3FC0, 0x3E80, 0xFE41,
    0xFA01, 0x3AC0, 0x3B80, 0xFB41, 0x3900, 0xF9C1, 0xF881, 0x3840,
    0x2800, 0xE8C1, 0xE981, 0x2940, 0xEB01, 0x2BC0, 0x2A80, 0xEA41,
    0xEE01, 0x2EC0, 0x2F80, 0xEF41, 0x2D00, 0xEDC1, 0xEC81, 0x2C40,
    0xE401, 0x24C0, 0x2580, 0xE541, 0x2700, 0xE7C1, 0xE681, 0x2640,
    0x2200, 0xE2C1, 0xE381, 0x2340, 0xE101, 0x21C0, 0x2080, 0xE041,
    0xA001, 0x60C0, 0x6180, 0xA141, 0x6300, 0xA3C1, 0xA281, 0x6240,
    0x6600, 0xA6C1, 0xA781, 0x6740, 0xA501, 0x65C0, 0x6480, 0xA441,
    0x6C00, 0xACC1, 0xAD81, 0x6D40, 0xAF01, 0x6FC0, 0x6E80, 0xAE41,
    0xAA01, 0x6AC0, 0x6B80, 0xAB41, 0x6900, 0xA9C1, 0xA881, 0x6840,
    0x7800, 0xB8C1, 0xB981, 0x7940, 0xBB01, 0x7BC0, 0x7A80, 0xBA41,
    0xBE01, 0x7EC0, 0x7F80, 0xBF41, 0x7D00, 0xBDC1, 0xBC81, 0x7C40,
    0xB401, 0x74C0, 0x7580, 0xB541, 0x7700, 0xB7C1, 0xB681, 0x7640,
    0x7200, 0xB2C1, 0xB381, 0x7340, 0xB101, 0x71C0, 0x7080, 0xB041,
    0x5000, 0x90C1, 0x9181, 0x5140, 0x9301, 0x53C0, 0x5280, 0x9241,
    0x9601, 0x56C0, 0x5780, 0x9741, 0x5500, 0x95C1, 0x9481, 0x5440,
    0x9C01, 0x5CC0, 0x5D80, 0x9D41, 0x5F00, 0x9FC1, 0x9E81, 0x5E40,
    0x5A00, 0x9AC1, 0x9B81, 0x5B40, 0x9901, 0x59C0, 0x5880, 0x9841,
    0x8801, 0x48C0, 0x4980, 0x8941, 0x4B00, 0x8BC1, 0x8A81, 0x4A40,
    0x4E00, 0x8EC1, 0x8F81, 0x4F40, 0x8D01, 0x4DC0, 0x4C80, 0x8C41,
    0x4400, 0x84C1, 0x8581, 0x4540, 0x8701, 0x47C0, 0x4680, 0x8641,
    0x8201, 0x42C0, 0x4380, 0x8341, 0x4100, 0x81C1, 0x8081, 0x4040,
  };

  unsigned short int crc = 0xffff;
  unsigned short int temp;
  int i;

  for (i = 0;  i < count;  i++) {
    crc ^= bytp[i];
    crc = (crc >> 8) ^ (crcTable [crc & 0xff]);
  }

  temp = (crc >>8) & 0xFF;
  crc = ((crc &0xff) << 8) | temp;
  return crc;
}

/*
 *********************************************************************
 * Given a pointer to some bytes and their length, return LRC
 *********************************************************************
 */

static VUINT8 vmblrc (VUINT8 *bp, int length) {
  VUINT8 cc = 0;
  int i;
  for (i = 0;  i < length;  i++)
    cc += bp[i];
  return((~cc) +1);
}

static VUINT8 vmblrc3 (VUINT8 slave, VUINT8 *bp, unsigned int length) {
  VUINT8 cc = slave;
  unsigned int i;
  for (i = 0;  i < length;  i++)
    cc += bp[i];
  return((~cc) +1);
}

/*
 *********************************************************************
 * vmbput()
 *
 * Put an unsigned short int swapped into a location
 *********************************************************************
 */

static void vmbput(unsigned char *uip, unsigned short int value) {
  *((unsigned char *) uip) = value >>8;
  *(((unsigned char *) uip) +1) = value &0xff;
}

/*
 *********************************************************************
 * Get an unsigned short int swapped from a location
 *********************************************************************
 */

static unsigned short int vmbget(unsigned char *ucp) {
  return((((unsigned short int) ucp[0]) <<8) + (unsigned short int) ucp[1]);
}

/*
 *********************************************************************
 * Conversions
 *********************************************************************
 */

#define BINTOASC(x) (((x) > 9) ? (x) +'7' : (x) +'0')

static int asctobin (VUINT8 *bin, VUINT8 *asc, unsigned n) {

  unsigned i;
  int d1;
  int d2;

#define M(d) do {				\
  if (*asc >= '0' && *asc <= '9')		\
    d = *asc - '0';				\
  else if (*asc >= 'A' && *asc <= 'F')		\
    d = *asc - 'A' + 10;			\
  else if (*asc >= 'a' && *asc <= 'f')		\
    d = *asc - 'a' + 10;			\
  else						\
    return FAILURE;				\
  asc++;					\
} while (0)
  
  for (i = 0; i < n; i++) {
    M (d1);
    M (d2);
    *bin++ = (d1 << 4) + d2;
  }

#undef M

  return SUCCESS;
}

/*
 *********************************************************************
 */

static int connectSocket (vp_t *h) {
  struct tcp_s *s = &h->tcp;
  if (s->socklib == NULL) {
    if ((s->socklib = vsocketInit ()) == NULL)
      return errNOMEM (h);
  }
  if (vsocketOpenClientSocket (s->socklib, h->name, s->port, &s->socket)
      != VSOCKET_OK)
    return socketError (h);
  return SUCCESS;
}

/*
 *********************************************************************
 */

static int connectUser (vp_t *h) {

  char *fileList[2];
  char driverBuffer[512];
  char *driver;

#ifdef VMS
  char ext[] = ""; /* Do not add extension, otherwise logical names
		      would not work. */
#else
  char ext[] = "." VSYS_LIB_SHR_EXT;
#endif

  if (h->user.driver != NULL && strchr (h->user.driver, '.') != NULL)
    driver = h->user.driver;
  else {
    driver = driverBuffer;
    if (h->user.driver == NULL)
      (void) sprintf (driverBuffer, "%s%s", h->name, ext);
    else
      (void) sprintf (driverBuffer, "%s%s", h->user.driver, ext);
  }

  fileList[0] = driver;
  fileList[1] = NULL;

  h->user.connect =
    (int (*)(vp_t*)) vsys_lib_find_routine ("userConnect", fileList);
  h->user.transact =
    (int(*)(vp_t*,int,unsigned,unsigned))
    vsys_lib_find_routine ("userTransact", fileList);
  h->user.disconnect =
    (int (*)(vp_t*)) vsys_lib_find_routine ("userDisconnect", fileList);

  if (h->user.connect == NULL ||
      h->user.transact == NULL) {
    return errINTERR (h);
  }

  if (h->user.connect (h) != 0)
    return errINTERR (h);

  return SUCCESS;
}

/*
 *********************************************************************
 */

static int getSerialResponse (vp_t *h, unsigned expectedLen, int *ret_slave) {

  VUINT8 *response = USER_RESPONSE_BUFFER (h);
  VUINT8 *request = USER_REQUEST_BUFFER (h);
  unsigned normalLen; /* Length of a normal reply, including protocol
			 stuff (CS, CRLF, etc). */
  unsigned exceptionLen; /* Length of an exception reply. */
  unsigned bytesRead = 0; /* Total number of bytes read. */
  unsigned actualLen;	/* Length of data, without unit, checksums, etc. */
  VUINT8 *msg;	/* Buffer for slRead. */
  bool errFlag = 0;
 /*  int ret_hldr; */
  bool ascii = h->type == VMB_TYPE_ASCII || h->type == VMB_TYPE_TCP_ASCII;
  bool tcp = h->type == VMB_TYPE_TCP_RTU || h->type == VMB_TYPE_TCP_ASCII;

  /*
   * A response is either a normal response or an exception.
   * All exceptions seem to have length == 2 (EL). They are shorter
   * than normal responses. We try to read the entire message.
   * Depending on platform, slRead will either return with less bytes
   * read than expected, in which case we have a chance to check
   * if this is an exception, or will time out, which is also OK
   * since normally there should be no exceptions.
   */
  
#define EL 2

  if (ascii) {
    exceptionLen = 1 /*:*/ + 2 /*unit*/ + EL * 2 + 2 /*CS*/ + 2 /*CRLF*/;
    normalLen = exceptionLen + (expectedLen - EL) * 2;
    msg = h->asciiBuffer;
  }
  else /* rtu */ {
    exceptionLen = 1 + EL + 2;
    normalLen = exceptionLen + (expectedLen - EL);
    msg = response - 1;
  }

  while (1) {

    VUINT8* ptr = msg + bytesRead;
    unsigned bytesToRead = normalLen - bytesRead;
    unsigned n;

    if (tcp) {
      vsocket_t *socket = h->tcp.socket;
      if (vsocketWait (socket, &h->tcp.timeout, 0) != VSOCKET_OK ||
	  vsocketRead (socket, ptr, bytesToRead, &n) != VSOCKET_OK) {
	if (h->debug) {
	  errFlag = 1;
	  break;
	}
	return socketError (h);
      }
    }
    else {
      if (slRead (h->sl, ptr, bytesToRead, &n) != SL_OK) {
	if (h->debug) {
	  errFlag = 1;
	  break;
	}
	return slError (h);
      }
    }
    bytesRead += n;
    if (bytesRead == normalLen)
      break;



    /* Check if what we have read so far looks like an exception. */
    if (bytesRead == exceptionLen) {
      if (ascii) {
	/* FIXME: Chapter 1 Modbus Protocol, page 5, Section "Exception"
	   says that CRLF is optional. Our ascii mode would not work in
	   that case. */
	if (msg[exceptionLen - 2] == '\r' &&
	    msg[exceptionLen - 1] == '\n')
	  break;
      }
      else /* rtu */ {
	if (request[0] != response[0]) {
	  break;
	}
      }
    }
  }

  if (h->debug) {
    (void) printf ("Expected length = %u, actual length = %u\n",
		   normalLen, bytesRead);
    hexdump ("Reply", msg, bytesRead);
    if (errFlag) {
      return tcp ?
	socketError (h) :
	slError (h);
    }
  }

  *ret_slave = (int)msg[0];
  /*   vsys_msg_printf(0, "Here %x %x", request - 1, msg[0]); */

  if (ascii) {
    actualLen = (bytesRead - 7) / 2;
    if (msg[0] != ':') return errPROTOCOL (h);
    if (msg[bytesRead - 2] != '\r' ||
	msg[bytesRead - 1] != '\n')
      return errPROTOCOL (h);
    if (asctobin (response - 1, msg + 1, actualLen + 2) != SUCCESS)
      return errPROTOCOL (h);
    if (vmblrc (response - 1, actualLen + 1) != response [actualLen])
      return errPROTOCOL (h);
  }
  else /* rtu */ {
    actualLen = bytesRead - 3;
    if (vmbget (response + actualLen) != vmbcrc16 (msg, 1 + actualLen)) {
    vsys_msg_printf(0, "Response %d crc %d\n", vmbget (response + actualLen),
      vmbcrc16 (msg, 1 + actualLen));
      vsys_msg_printf(0, "error 505: CRC error\n");
      return errPROTOCOL (h);    
    }
  }

#undef EL

  return SUCCESS;
}

/*
 *********************************************************************
 */

static int getSocketResponse (vp_t *h, unsigned expectedLen) {

  vsocket_t *socket = h->tcp.socket;
  VUINT8 *response = USER_RESPONSE_BUFFER (h);
  VUINT8 *request = USER_REQUEST_BUFFER (h);
  VUINT8 *msg = response - 7;
  VUINT8 *ptr = msg;
  unsigned bytesToRead = expectedLen + 7;
  unsigned bytesRead = 0;
  unsigned actualLen;

  while (1) {
    unsigned n;
    if (vsocketWait (socket, &h->tcp.timeout, 0) != VSOCKET_OK ||
	vsocketRead (socket, ptr, bytesToRead, &n) != VSOCKET_OK)
      return socketError (h);
    bytesRead += n;
    bytesToRead -= n;
    if (bytesToRead == 0) break;
    if (bytesRead >= 9 && request[0] != response[0]) break;
    ptr += n;
  }

  actualLen = bytesRead - 7;
  if (msg[4] != 0 || msg[5] != actualLen + 1) return errPROTOCOL (h);

  return SUCCESS;
}

/*
 *********************************************************************
 */

static int sendRequest (vp_t *h, int slave, unsigned int requestLen) {

  VUINT8 *request = USER_REQUEST_BUFFER (h);
  unsigned msgLen = 0;
  VUINT8 *msg = NULL;
  unsigned i;

  if (! h->connected) {
    vsys_wait (1000); /* Do not generate to many error messages. */
    if (vmbConnect (h, h->name) != SUCCESS)
      return FAILURE;
  }

  switch (h->type) {
  case VMB_TYPE_TCP:
    msg = request - 7;
    msg[0] = 0; /* Transaction ID */
    msg[1] = 0; /* Transaction ID */
    msg[2] = 0; /* Protocol ID */
    msg[3] = 0; /* Protocol ID */
    msg[4] = 0; /* Length */
    msg[5] = requestLen + 1;
    msg[6] = (VUINT8) slave;
    msgLen = requestLen + 7;
    if (vsocketWriteAll (h->tcp.socket, msg, msgLen) != VSOCKET_OK)
      return socketError (h);
    if (h->debug) 
      hexdump ("Request", msg, msgLen);
    break;

  case VMB_TYPE_TCP_RTU:
  case VMB_TYPE_RTU:
    /* Simply move and checksum. */
    msg = request - 1;
    msg[0] = (VUINT8) slave;
    vmbput (&request[requestLen], vmbcrc16 (msg, requestLen + 1));
    msgLen = requestLen + 3;

    if (h->type == VMB_TYPE_RTU) {
      if (slWriteAll (h->sl, msg, msgLen) != SL_OK)
	return slError (h);
    }
    else {
      if (vsocketWriteAll (h->tcp.socket, msg, msgLen) != VSOCKET_OK)
	return socketError (h);
    }
    if (h->debug) 
      hexdump ("Request", msg, msgLen);
    break;

  case VMB_TYPE_TCP_ASCII:
  case VMB_TYPE_ASCII: {
    VUINT8 csum = vmblrc3 ((VUINT8) slave, request, requestLen);
    VUINT8 *p = h->asciiBuffer;
    msg = p;

    *p++ = ':';
    *p++ = BINTOASC (slave >> 4);
    *p++ = BINTOASC (slave & 0xf);

    for (i = 0; i < requestLen; i++) {
      *p++ = BINTOASC (request[i] >> 4);
      *p++ = BINTOASC (request[i] & 0xf);
    }
    *p++ = BINTOASC (csum >> 4);
    *p++ = BINTOASC (csum & 0xf);
    *p++ = 13;
    *p++ = 10;
    msgLen = requestLen * 2 + 3 + 4;

    if (h->type == VMB_TYPE_ASCII) {
      if (slWriteAll (h->sl, msg, msgLen) != SL_OK)
	return slError (h);
    }
    else {
      if (vsocketWriteAll (h->tcp.socket, msg, msgLen) != VSOCKET_OK)
	return socketError (h);
    }
    break;
  }

  default:
    return errINTERR (h);
  }

  return SUCCESS;
}

/*
 *********************************************************************
 */

int vmbTransact (vp_t *h, int slave, unsigned requestLen,
		 unsigned expectedResponseLen, int *ret_slave) {

  int status;

  if (requestLen > 255 && expectedResponseLen > 255)
    return errINVARG (h);

  switch (h->type) {
  case VMB_TYPE_USER:
    status = h->user.transact (h, slave, requestLen, expectedResponseLen);
    break;
  case VMB_TYPE_TCP:
    status = sendRequest (h, slave, requestLen);
    if (status == SUCCESS)
      status = getSocketResponse (h, expectedResponseLen);
    break;
  case VMB_TYPE_ASCII:
  case VMB_TYPE_RTU:
  case VMB_TYPE_TCP_ASCII:
  case VMB_TYPE_TCP_RTU:
    if (control_design)
      slFlushAll(h->sl); 
    status = sendRequest (h, slave, requestLen);
    if (status == SUCCESS)
      status = getSerialResponse (h, expectedResponseLen, ret_slave);
    break;
  default:
    return errINTERR (h);
  }

  return status;
}

/*
 *********************************************************************
 */
void set_control_design(int num) {

  control_design = num;

}

void set_show_decimal(int num) {

  showDecimal = num;

}

int vmbConnect (vp_t *h, char *name) {

  if (name == NULL)
    h->name[0] = 0;
  else
    (void) vsys_strncpy (h->name, name, sizeof h->name);

  switch (h->type) {
  case VMB_TYPE_ASCII:
  case VMB_TYPE_RTU:
    if (slOpen (h->sl, name) != SL_OK)
      return slError (h);
    break;
  case VMB_TYPE_TCP:
  case VMB_TYPE_TCP_ASCII:
  case VMB_TYPE_TCP_RTU:
    if (connectSocket (h) != SUCCESS)
      return FAILURE;
    break;
  case VMB_TYPE_USER:
    if (connectUser (h) != SUCCESS)
      return FAILURE;
    break;
  default:
    return errINTERR (h);
  }
  
  h->connected = 1;
  return SUCCESS;
}

/*
 *********************************************************************
 */

int vmbDisconnect (vp_t *h) {

  if (h->connected) {

    h->connected = 0;

    switch (h->type) {

    case VMB_TYPE_ASCII:
    case VMB_TYPE_RTU:
      (void) slClose (h->sl);
      break;

    case VMB_TYPE_TCP_ASCII:
    case VMB_TYPE_TCP_RTU:
    case VMB_TYPE_TCP:
      if (h->tcp.socket != NULL) {
	(void) vsocketClose (h->tcp.socket);
	h->tcp.socket = NULL;
      }
      break;

    case VMB_TYPE_USER:
      if (h->user.disconnect != NULL) {
	if (h->user.disconnect (h) != 0)
	  return errINTERR (h);
      }
      break;

    default:
      return errINTERR (h);
    }
  }

  return SUCCESS;
}

/*
 *********************************************************************
 */

char * vmbStrerror (vp_t *h) {
  return h->errstr;
}

/*
 *********************************************************************
 */

void* vmbGetPtr (vp_t *h, int ptrType) {
  switch (ptrType) {
  case VMB_P_USER_REQUEST_BUFFER: return USER_REQUEST_BUFFER (h);
  case VMB_P_USER_RESPONSE_BUFFER: return USER_RESPONSE_BUFFER (h);
  }
  return NULL;
}

/*
 *********************************************************************
 */

int vmbGet (vp_t *h, int option, void *value) {

  bool connected = h->connected;
  sl_t *sl = h->sl;
  
  switch (option) {
  case VMB_O_BAUD:
    if (! connected || h->sl == NULL) goto err;
    if (slGetBaud (sl, value) != SL_OK)
      return slError (h);
    break;
  case VMB_O_PARITY:
    if (! connected || h->sl == NULL) goto err;
    if (slGetParity (sl, value) != SL_OK)
      return slError (h);
    break;
  case VMB_O_STOP:
    if (! connected || h->sl == NULL) goto err;
    if (slGetStop (sl, value) != SL_OK)
      return slError (h);
    break;
  default:
    return errINVARG (h);
  }

  return SUCCESS;

 err:
  return errINVARG (h); /* h->connected == 0 */
}

/*
 *********************************************************************
 */

int vmbSet (vp_t *h, int option, void *value) {

  bool connected = h->connected;
  sl_t *sl = h->sl;
  
  switch (option) {
  case VMB_O_BAUD:
    if (connected) goto err;
    if  (h->sl != NULL && slSetBaud (sl, (int) value) != SL_OK)
      return slError (h);
    break;
  case VMB_O_DEBUG:
    h->debug = (int) value;
    if (h->sl != NULL && slSetDebug (h->sl, (int) value) != SL_OK)
      return slError (h);
    break;
  case VMB_O_DRIVER:
    if (connected) goto err;
    if (h->user.driver != NULL) VMEM_FREE (h->user.driver);
    if (value != NULL) {
      h->user.driver = VMEM_STRDUP (value);
      if (h->user.driver == NULL) return errNOMEM (h);
    }
    break;
  case VMB_O_ERROR:
     (void) stringError (h, value);
    break;
  case VMB_O_PARITY:
    if (connected) goto err;
    if  (h->sl != NULL && slSetParity (sl, (int) value) != SL_OK)
      return slError (h);
    break;
  case VMB_O_PORT:
    /* normally it is VMB_HYTEC_PORT, but for debugging purpose
       we may use another one (to avoid running our debugging server
       from root, for example) */
    if (connected) goto err;
    h->tcp.port = (int) value;
    break;
  case VMB_O_STOP:
    if (connected) goto err;
    if  (h->sl != NULL && slSetStop (sl, (int) value) != SL_OK)
      return slError (h);
    break;
  case VMB_O_TYPE:
    if (connected) goto err;
    h->type = (int) value;
    break;
  case VMB_O_RTS:
    if (connected) goto err;
    if  (h->sl != NULL && slSetRts (sl, (int) value) != SL_OK)
      return slError (h);
    break;
  case VMB_O_TURNAROUND:
    if (connected) goto err;
    if  (h->sl != NULL && slSetTurnaround (sl, (int) value) != SL_OK)
      return slError (h);
    break;
  case VMB_O_TIMEOUT:
    if (connected) goto err;
    h->tcp.timeout.t_sec = (unsigned) value;
    h->tcp.timeout.t_nsec = 0;
    if (h->sl != NULL && slSetTimeout (sl, (unsigned) value) != SL_OK)
      return slError (h);
    break;
  default:
    return errINVARG (h);
  }

  return SUCCESS;

 err:
  return errINVARG (h); /* h->connected != 0 */
}

/*
 *********************************************************************
 */

vp_t * vmbAlloc (VUINT8 **requestBuf, unsigned *requestBufSize,
		 VUINT8 **responseBuf, unsigned *responseBufSize) {
  vp_t *h = VMEM_MALLOC0 (sizeof *h);
  if (h != NULL) {
    if ((h->sl = slInit ()) == NULL) {
      VMEM_FREE (h);
      h = NULL;
    }
  }
  if (h != NULL) {
    h->tcp.port = VMB_HYTEC_PORT;
    h->type = VMB_TYPE_RTU;
    h->debug = 0;
    h->tcp.timeout.t_sec = 5;
    h->tcp.timeout.t_nsec = 0;
    *requestBuf = USER_REQUEST_BUFFER (h);
    *responseBuf = USER_RESPONSE_BUFFER (h);
    *requestBufSize = USER_BUFFER_SIZE;
    *responseBufSize = USER_BUFFER_SIZE;
  }
  return h;
}
