#include <ctype.h>              /* for toupper() */
#include <stdio.h>
#include "vdb_structure.h"      /* for vista_chix */
#include "vdb_convert.h"        /* for conversion_record */
#include "vdb_descrip.h"        /* for vdb_descrip_s */
#include <vsys_routines.h>
#include <vdb_routines.h>
#include <vdb_errors.h>
#include <vdb_class.h>
 
int main (int argc, char *argv) {

}
int lovington_limits (
    vista_chix  *Chix,           /* Vista chix for channel */
    int         *numparams,     /* number of cparams passed */
    float       *Params,        /* pointer to array of cparams */
    int         *numvals,       /* number of values passed */
    float       *invalue,     /* pointer to array of unconverted values */
    float       *outvalue)     /* pointer to array of converted values */
{
  int     i = 0, x = 0;
  struct vdb_descrip_v desc;
  VDB_DATA_ITEM_LIST dlist[40]; 
  char       chan_name[12000];
  float llim = 0, ulim = 0;
  float prevdata = 0;

  /*Get this channel name and cparam1 (the stored previous value), 
   along with upper and lower display limits */
  dlist[0].chix = *Chix; 
  dlist[0].event_class_number= VCLS_N_INFO; 
  dlist[0].event_field_mask= VFLD_M_CHNAME; 
  dlist[0].index= 0; 
  dlist[0].dtype= VDB_DTYPE_STRING; 
  dlist[0].size= 255; 
  dlist[0].data_addr= chan_name; 
  
  dlist[1].chix = *Chix; 
  dlist[1].event_class_number = VCLS_N_CONVERSION; 
  dlist[1].event_field_mask = VFLD_M_CPARAM1; 
  dlist[1].dtype = VDB_DTYPE_FLOAT; 
  dlist[1].size = sizeof(float); 
  dlist[1].data_addr = &prevdata; 

  dlist[2].chix = *Chix; 
  dlist[2].event_class_number = VCLS_N_LIMIT; 
  dlist[2].event_field_mask = VFLD_M_UD; 
  dlist[2].dtype = VDB_DTYPE_FLOAT; 
  dlist[2].size = sizeof(float); 
  dlist[2].data_addr = &ulim; 

  dlist[3].chix = *Chix; 
  dlist[3].event_class_number = VCLS_N_LIMIT; 
  dlist[3].event_field_mask = VFLD_M_LD; 
  dlist[3].dtype = VDB_DTYPE_FLOAT; 
  dlist[3].size = sizeof(float); 
  dlist[3].data_addr = &llim; 
  
  vdb_zero_chix(&(dlist[4].chix)); 
  
  vdb_get_item (dlist); 

  //  vsys_msg_printf(0, "CONV:channel %s with value %f, ud %f, ld %f, prevvalue %f\n", chan_name, *invalue, ulim, llim, prevdata);

  /* If the last value was outside the limit, and this one is inside, clear the
     IOError. If the last value was inside the limit, and this one is outside,
     set the IOError. Else, do nothing.*/
  if ((prevdata > ulim && *invalue < ulim) || 
      (prevdata < llim && *invalue > llim)) {
      int value = 0;
      VSYS_INTEGER_FILL (desc, value, sizeof value);
      //  vsys_msg_printf(0, "Clearing IO error!");
      vdb_put (Chix, VFLD_M_ENTLVE, VFLD_M_IO_ERROR, &desc);
  } else if ((prevdata < ulim && *invalue > ulim) || 
	     (prevdata > llim && *invalue < llim)) {
    int value = 1;
    VSYS_INTEGER_FILL (desc, value, sizeof value);
    //  vsys_msg_printf(0, "Setting IO error!");
    vdb_put (Chix, VFLD_M_ENTLVE, VFLD_M_IO_ERROR, &desc);
  }

  /* This part sets the "previous value" to the current one, in preparation
   for the next value*/
  dlist[0].chix = *Chix; 
  dlist[0].event_class_number = VCLS_N_CONVERSION; 
  dlist[0].event_field_mask = VFLD_M_CPARAM1; 
  dlist[0].dtype = VDB_DTYPE_FLOAT; 
  dlist[0].size = sizeof(float); 
  dlist[0].data_addr = invalue; 
		
  vdb_zero_chix(&(dlist[1].chix)); 
  
  vdb_put_item (dlist); 
			     
  return (1); /* 1 == success */
}
