/******************************************************************************
*        ____
*      _-    -_    _------_
*     -        -_--        -_
*          _--------------_   -_
* 	_---              -_
*      _-                 ----___
*                         ---___
*      Copyright (c) 2003-2004 Vista Control Systems, Inc.
*      All rights reserved
*      176 Central Park Square * Los Alamos, N.M. 87544 * 505-662-2484
*
*      Filename: vhytec/scanner_lib.c
*
**********************************************************************/

#include <stdarg.h>
#include <signal.h>

#define VERR_FILE_ID "LB"
extern "C" {
#include "scanner_lib.h"
}
static struct {
  VthreadMutexEx logMutex;	/* Mutex for serializing access to the
				   log file/status channel. */
  vista_chix statusChannelChix;
  unsigned eventQueueSize;
  char* statusChannelName;
  char* programName;
  char* versionString;
  int01 banner;			/* Display startup banner. */
  int01 noline;			/* Do not print __LINE__ number (for
				   autotests). */
  bool statusChannel;		/* We have a valid status channel. */
  bool logTS;			/* Timestamp all log file records. This option
				   is enabled by default. The /nots option
				   is often used in autotests. */
  char statusChannelBuff[512];	/* Buffer for writing into status channel. */
} data;

/*
 *********************************************************************
 */

#define errorSTATUSPUT(c) vscannerLogError ("Error writing to status channel")

#define fatalNOSTSCHAN() vscannerLogFatal ("Status channel not found")
#define fatalINVSTSTYPE() vscannerLogFatal ("Invalid status channel type")

/*
 *********************************************************************
 */

static void logVaList (char* format, va_list args) {

  if (data.logMutex.initialized)
    (void) vthreadMutexLockEx (&data.logMutex);

  if (data.statusChannel) {
    /* We do not really need a mutex to write to the status channel - vaccess
       takes care of that. But we do need a mutex to access the buffer. */
    if (vsys_vsnprintf (data.statusChannelBuff, sizeof data.statusChannelBuff,
			format, args) == VSYS_OK) {
      if (vdbc_sput (&data.statusChannelChix, data.statusChannelBuff)
	  == VDB_ERR_OK)
	goto ret;

      data.statusChannel = 0;				/*NYCV*/
      if (data.logMutex.initialized)			/*NYCV*/
	(void) vthreadMutexUnlockEx (&data.logMutex);	/*NYCV*/
      (void) errorSTATUSPUT (&data.statusChannelChix);	/*NYCV*/
      if (data.logMutex.initialized)			/*NYCV*/
	(void) vthreadMutexLockEx (&data.logMutex);	/*NYCV*/
    }
  }

  if (data.logTS) {
    char asctim[50];
    vtime bintim;
    (void) vsys_get_time (&bintim);				/*NCV*/
    asctim[0] = 0;						/*NCV*/
    (void) vsysc_asctime (asctim, sizeof asctim, &bintim);	/*NCV*/
    (void) printf ("%s: ", asctim);				/*NCV*/
  }

  (void) vprintf (format, args);
  (void) printf ("\n");
  (void) fflush (stdout);

 ret:
  if (data.logMutex.initialized)
    (void) vthreadMutexUnlockEx (&data.logMutex);
}

static void logVaListNoprint (char* format, va_list args) {

  if (data.logMutex.initialized)
    (void) vthreadMutexLockEx (&data.logMutex);

  if (data.statusChannel) {
    /* We do not really need a mutex to write to the status channel - vaccess
       takes care of that. But we do need a mutex to access the buffer. */
    if (vsys_vsnprintf (data.statusChannelBuff, sizeof data.statusChannelBuff,
			format, args) == VSYS_OK) {
      if (vdbc_sput (&data.statusChannelChix, data.statusChannelBuff)
	  == VDB_ERR_OK)
	goto ret;

      data.statusChannel = 0;				/*NYCV*/
      if (data.logMutex.initialized)			/*NYCV*/
	(void) vthreadMutexUnlockEx (&data.logMutex);	/*NYCV*/
      (void) errorSTATUSPUT (&data.statusChannelChix);	/*NYCV*/
      if (data.logMutex.initialized)			/*NYCV*/
	(void) vthreadMutexLockEx (&data.logMutex);	/*NYCV*/
    }
  }

  /*  if (data.logTS) {
    char asctim[50];
    vtime bintim;
    (void) vsys_get_time (&bintim);				
    asctim[0] = 0;						
    (void) vsysc_asctime (asctim, sizeof asctim, &bintim);
    (void) printf ("%s: ", asctim);			
  }
*/
  /*  (void) vprintf (format, args);
  (void) printf ("\n");
  (void) fflush (stdout);
  */
 ret:
  if (data.logMutex.initialized)
    (void) vthreadMutexUnlockEx (&data.logMutex);
}

/*
 *********************************************************************
 */

void vscannerLogMsg (char* format, ...) {
  va_list args;
  va_start (args, format);
  logVaList (format, args);
  va_end (args);
}

void vscannerLogMsgNoprint (char* format, ...) {
  va_list args;
  va_start (args, format);
  logVaListNoprint (format, args);
  va_end (args);
}

/*
 *********************************************************************
 */

void vscannerExitFailure (void) {
  vscannerLogMsg ("Exiting (EXIT_FAILURE)");
  exit (EXIT_FAILURE);
} /*NCV*/

/*
 *********************************************************************
 * This routine never returns. But it is defined as 'int' to make
 * it more convenient to use it in 'main'.
 *********************************************************************
 */

int vscannerExitSuccess (void) {
  vscannerLogMsg ("Exiting (EXIT_SUCCESS)");
  exit (EXIT_SUCCESS);
  return EXIT_SUCCESS;
}

/*
 *********************************************************************
 */

void vscannerLogFatal (char* format, ...) {
  va_list args;
  va_start (args, format);
  logVaList (format, args);
  va_end (args);
  vscannerExitFailure ();
} /*NCV*/

/*
 *********************************************************************
 */

int vscannerLogError (char* format, ...) {
  va_list args;
  va_start (args, format);
  logVaList (format, args);
  va_end (args);
  return VSCANNER_FAILURE;
}

/*
 *********************************************************************
 */

void vscannerFatalInterr_LINE (char* line) {	/*NCV*/
  vscannerLogFatal ("Internal error %s", line);	/*NCV*/
}						/*NCV*/

/*
 *********************************************************************
 */

void vscannerInitLogMutex (void) {
  if (vthreadMutexInitEx (&data.logMutex, NULL) != VTHREAD_OK)
    vscannerFatalInterr ();	/*NCV*/
}

/*
 *********************************************************************
 */

void vscannerBanner (void) {
  if (data.banner) 
    vscannerLogMsg ("Vsystem %s Version %s",			/*NCV*/
		    data.programName, data.versionString);	/*NCV*/
}

/*
 *********************************************************************
 */

void vscannerSetNameAndVersion (char* name, char* version) {
  data.programName = name;
  data.versionString = version;
}

/*
 *********************************************************************
 * LN instead of LINE because the name is too long on OpenVMS I64
 *********************************************************************
 */

void vscannerFatalVdbStatusError_LN (vista_chix* chix, int status,
				     char* line) {
  if (data.noline)
    vscannerLogFatal ("VDB error 0x%x, chix = %x", status, *(int*) chix);
  else								/*NCV*/
    vscannerLogFatal ("VDB error 0x%x at %s, chix = %x",	/*NCV*/
		      status, line, *(int*) chix);		/*NCV*/
}								/*NCV*/

void vscannerFatalVdbChixError_LINE (vista_chix* chix, char* line) {
  int status = 0;
  (void) vdb_error_status (chix, &status);
  vscannerFatalVdbStatusError_LN (chix, status, line);
}	/*NCV*/

/*
 *********************************************************************
 */

int vscannerDbChannelType (vista_chix* chix) {
  short type, in, soft;
  if (vdb_channel_type (chix, &type, &in, &soft) != VDB_ERR_OK)
    vscannerFatalVdbChixError (chix); /*NCV*/
  return type;
}

/*
 *********************************************************************
 */

int vscannerDbIn (vista_chix* chix) {
  short type, in, soft;
  if (vdb_channel_type (chix, &type, &in, &soft) != VDB_ERR_OK)
    vscannerFatalVdbChixError (chix); /*NCV*/
  return in;
}

/*
 *********************************************************************
 */

void vscannerInitStatusChannel (void) {
  char* name = data.statusChannelName;
  if (name != NULL) {
    vista_chix* chix = &data.statusChannelChix;
    if (vdbc_channel_index (name, chix) != VDB_ERR_OK)
      fatalNOSTSCHAN (); /*NCV*/
    if (vscannerDbChannelType (chix) != VDB_TYPE_STRING)
      fatalINVSTSTYPE ();
    data.statusChannel = 1;
    vscannerBanner ();
  }
}

/*
 *********************************************************************
 */

int vscannerDefineStandardVcliGroup (vcli_t* cli) {
  return vcliDefineGroup
    (cli, "standard",
     "q banner default 1", &data.banner,
     "q eventQueueSize $uint", &data.eventQueueSize,
     "q _noline nohelp", &data.noline,
     "q statusChannel $string", &data.statusChannelName,
     "q ts default 1 nohelp", &data.logTS,
     NULL);
}

/*
 *********************************************************************
 */

int01 vscannerGetNoline (void) {
  return data.noline;
}

/*
 *********************************************************************
 */

void vscannerIgnoreSigfpe (void) {
  if (signal (SIGFPE, SIG_IGN) == SIG_ERR)
    vscannerFatalInterr ();	/*NCV*/
}

/*
 *********************************************************************
 */

void vscannerSetEventQueueSize (void) {
  if (data.eventQueueSize != 0)
    if (vdb_tune_set (data.eventQueueSize, VDB_TUNE_EEF_ITEM) != VDB_SUCCESS)
      vscannerFatalInterr ();	/*NCV*/
}
