/******************************************************************************
*        ____
*      _-    -_    _------_
*     -        -_--        -_
*          _--------------_   -_
* 	_---              -_
*      _-                 ----___
*                         ---___
*      Copyright (c) 2001 Vista Control Systems, Inc.
*      All rights reserved
*      176 Central Park Square * Los Alamos, N.M. 87544 * 505-662-2484
*
*      Filename: vhytec/setpriority.c
*
**********************************************************************/

/*
 * Program to get/set policy and priority of the specified process.
 * Author: Eugene Zharkov
 * Started: 18-Jul-2000
 */

#include <stdio.h>
#include <stdlib.h>
#include <sched.h>
#include <errno.h>
#include <sys/resource.h>

#define RRR(s) rrr (__LINE__, s)

static void rrr (int line, char *string) {
  perror (string);
  exit (EXIT_FAILURE);
}

static void get (int pid) {
  struct sched_param p;
  int policy;
  int priority;
  if (sched_getparam (pid, &p) != 0) RRR ("sched_getparam");
  policy = sched_getscheduler (pid);
  if (policy < 0) RRR ("sched_getscheduler");
  errno = 0;
  priority = getpriority (PRIO_PROCESS, pid);
  if (errno != 0) RRR ("getpriority");
  if (policy == SCHED_OTHER && p.sched_priority != 0)
    RRR ("other!=0");
  if (policy != SCHED_OTHER)
    priority = p.sched_priority;
  printf ("Policy/Priority %d/%d\n", policy, priority);
}

static void set (int pid, int policy, int priority) {
  struct sched_param p;
  p.sched_priority = priority;
  if (policy == SCHED_OTHER) {
    if (setpriority (PRIO_PROCESS, pid, priority) != 0)
      RRR ("setpriority");
  }
  else {
    if (sched_setscheduler (pid, policy, &p) != 0)
      RRR ("sched_setscheduler");
  }
}

static void usage (void) {
  printf ("Usage:\n");
  printf ("  cmd pid\n");
  printf ("  cmd pid policy priority\n");
  exit (EXIT_FAILURE);
}

int main (int argc, char **argv) {

  if (argc < 2)
    usage ();
  else {
    int pid = atoi (argv[1]);
    if (argc == 2)
      get (pid);
    else if (argc != 4)
      usage ();
    else {
      int policy = atoi (argv[2]);
      int priority = atoi (argv[3]);
      if (policy != SCHED_OTHER &&
	  policy != SCHED_FIFO &&
	  policy != SCHED_RR) {
	printf ("Invalid policy\n");
	exit (EXIT_FAILURE);
      }
      set (pid, policy, priority);
    }
  }
  return EXIT_SUCCESS;
}
