/******************************************************************************
*        ____
*      _-    -_    _------_
*     -        -_--        -_
*          _--------------_   -_
* 	_---              -_
*      _-                 ----___
*                         ---___
*      Copyright (c) 2000-2001 Vista Control Systems, Inc.
*      All rights reserved
*      176 Central Park Square * Los Alamos, N.M. 87544 * 505-662-2484
*
*      Filename: vhytec/setscanrate.c
*
**********************************************************************/

/*
 * A program to set the scan rate of multiple channel
 * Author: Eugene Zharkov
 * Started: 30-Nov-2000
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <vsys_version.h>
#include <vsys_errors.h>
#include <vsys_time.h>
#include <vsys_cli.h>
#include <vdb_structure.h>
#include <vdb_routines.h>
#include <vdb_class.h>
#include <vdb_errors.h>

#include "version.h"

static char *programName = "setScanRate";
static char commandName[] = "setscanrate";
static char versionString[] = VSYS_VERSION_STRING "." VERSION_STRING;

typedef int bool;

static struct {
  char *template;
  unsigned scanRate;
} data;

/*
 *********************************************************************
 */

static void tsPrint (char *format, ...) {
  va_list args;
  char asctim[50];
  vtime bintim;
  vsys_get_time (&bintim);
  asctim[0] = 0;
  vsysc_asctime (asctim, sizeof (asctim), &bintim);
  printf ("%s: ", asctim);
  va_start (args, format);
  vprintf (format, args);
  printf ("\n");
  va_end (args);
}

/*
 *********************************************************************
 */

#define RRR() rrr (__LINE__)
#define FIXME() rrr (__LINE__)

static void rrr (int line) {
  tsPrint ("vhytec/server: error at %d", line);
  exit (EXIT_FAILURE);
}

/*
 *********************************************************************
 */

static void parseCommandLine (int argc, char *argv[]) {

  vcli_t * cli;

  if ((cli = vcliInit ()) == NULL) RRR ();

  if (vcliSetTitle (cli, programName) != VCLI_OK ||
      vcliSetVersion (cli, versionString) != VCLI_OK ||
      vcliSetCommandName (cli, commandName) != VCLI_OK ||
      vcliEnableSortedHelp (cli) != VCLI_OK ||
      vcliVparse
      (cli,
       argc, argv,
       "p channelNameTemplate $string", &data.template,
       "p scanRate $uint", &data.scanRate,
       NULL) != VCLI_OK) {
    vcliPerror (cli, NULL);
    RRR ();
  }
}

/*
 *********************************************************************
 */

static void vdbErr (vista_chix *chix, char *routine) {
  int status = 0;
  vdb_error_status (chix, &status);
  printf ("VDB ERROR 0x%x from %s\n", status, routine);
  exit (EXIT_FAILURE);
}

/*
 *********************************************************************
 */

int main (int argc, char *argv[]) {

  struct vdb_descrip_v desc;
  vista_chix chix;
  unsigned numChannels = 0;
  unsigned oldScanRate = 0;
  unsigned value;
  bool differentOldScanRates = 0;

  parseCommandLine (argc, argv);

  vdb_zero_chix (&chix);
  VSYS_INTEGER_FILL (desc, value, sizeof value);

  while (vdbc_search (data.template, &chix) == VDB_ERR_OK) {

    numChannels++;

    if (vdb_get (&chix, VCLS_M_HARDWARE, VFLD_M_HPARAM2, &desc) != VDB_ERR_OK)
      vdbErr (&chix, "vdb_get");

    if (numChannels == 1)
      oldScanRate = value;
    else if (value != oldScanRate)
      differentOldScanRates = 1;
      
    value = data.scanRate;
    if (vdb_put (&chix, VCLS_M_HARDWARE, VFLD_M_HPARAM2, &desc) != VDB_ERR_OK)
      vdbErr (&chix, "vdb_put");
  }

  if (differentOldScanRates)
    printf ("Channels have different scan rates\n");
  else
    printf ("Old scan rate: %u\n", oldScanRate);
  printf ("New scan rate: %u\n", data.scanRate);
  printf ("Number of channels: %u\n", numChannels);
  return EXIT_SUCCESS;
}
