/******************************************************************************
*        ____
*      _-    -_    _------_
*     -        -_--        -_
*          _--------------_   -_
* 	_---              -_
*      _-                 ----___
*                         ---___
*      Copyright (c) 2001 Vista Control Systems, Inc.
*      All rights reserved
*      176 Central Park Square * Los Alamos, N.M. 87544 * 505-662-2484
*
*      Filename: vhytec/sliotest.c
*
**********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <vsys_version.h>
#include <vsys_cli.h>
#include "slio.h"

static char versionString[] = VSYS_VERSION_STRING ".1";
static char commandName[] = "sliotest";

typedef int int01;

enum {
  CMD_000 = 0,
  CMD_READ,
  CMD_WRITE
};

/* Parameters and qualifiers. */

static struct {
  unsigned length;
  char * device;
  char * ascii;
  unsigned binaryLength;
  unsigned * binary;
  int command;
} q;

/*
 *********************************************************************
 */

#define RRR() rrr (__LINE__)

static void rrr (int line) {
  fprintf (stderr, "\n*** error at %d\n", line);
  exit (EXIT_FAILURE);
}

static void cliError (vcli_t *cli) {
  vcliPerror (cli, NULL);
  exit (EXIT_FAILURE);
}

#define slError(sl) slError2 (sl, __LINE__)
static void slError2 (sl_t *sl, int line) {
  fprintf (stderr, "sl%d: %s\n", line, slStrerror (sl));
  exit (EXIT_FAILURE);
}

/*
 *********************************************************************
 */

static void hexdump (char *title, void *ptr, unsigned length) {
  unsigned char *c = ptr;
  unsigned i;
  printf ("DUMP %s", title);
  for (i = 0; i < length; i++) {
    if ((i % 16) == 0) printf ("\n%04X", i);
    printf (" %02X", c[i]);
  }
  printf ("\n");
}

/*
 *********************************************************************
 */

static void cmdRead (sl_t *sl) {
  char b[256];
  unsigned bytesRead;
  if (q.length >= sizeof b) RRR ();
  if (slRead (sl, b, q.length, &bytesRead) != SL_OK) slError (sl);
  hexdump ("b", b, bytesRead);
}

/*
 *********************************************************************
 */

static void cmdWrite (sl_t *sl) {
  char buff[256];
  unsigned length;
  void *msg;
  if (q.ascii != NULL) {
    msg = q.ascii;
    length = strlen (msg);
  }
  else {
    unsigned i;
    length = q.binaryLength;
    if (length >= sizeof buff) RRR ();
    for (i = 0; i < length; i++)
      buff[i] = q.binary[i];
    msg = buff;
  }
  if (slWriteAll (sl, msg, length) != SL_OK) slError (sl);
}

/*
 *********************************************************************
 */

int main (int argc, char **argv) {
  vcli_t *cli;
  sl_t *sl;
  if ((sl = slInit ()) == NULL) RRR ();
  if ((cli = vcliInit ()) == NULL) RRR ();

  if (vcliSetTitle (cli, "SLIOTEST Utility") != VCLI_OK ||
      vcliSetVersion (cli, versionString) != VCLI_OK ||
      vcliSetCommandName (cli, commandName) != VCLI_OK ||
      vcliEnableSortedHelp (cli) != VCLI_OK) {
    cliError (cli);
  }

  if (vcliVparse (cli,
		  argc, argv,
		  "define type command",
		  "k read syntax read", (void*) CMD_READ,
		  "k write syntax write", (void*) CMD_WRITE,
		  "p command type command", &q.command,
		  "define syntax read",
		  "p device $string", &q.device,
		  "p length $uint", &q.length,
		  "define syntax write",
		  "p device $string", &q.device,
		  "q ascii $string", &q.ascii,
		  "q binary list $uint", &q.binaryLength, &q.binary,
		  NULL) != VCLI_OK) {
    cliError (cli);
  }

  if (slOpen (sl, q.device) != SL_OK)
    slError (sl);

  switch (q.command) {
  case CMD_READ: cmdRead (sl); break;
  case CMD_WRITE: cmdWrite (sl); break;
  default: RRR ();
  }

  slDestroy (sl);
  vcliDestroy (cli);
  return EXIT_SUCCESS;
}
