/******************************************************************************
*        ____
*      _-    -_    _------_
*     -        -_--        -_
*          _--------------_   -_
* 	_---              -_
*      _-                 ----___
*                         ---___
*      Copyright (c) 2000-2004 Vista Control Systems, Inc.
*      All rights reserved
*      176 Central Park Square * Los Alamos, N.M. 87544 * 505-662-2484
*
*      Filename: vhytec.h
*
**********************************************************************/

#ifndef vhytec_h_LOADED
#define vhytec_h_LOADED

extern "C" {
#include "vsys_routines.h"
#include "vdb_structure.h"
}
#include "HytecIPAPI.h"
#include "Hy8411.h"
#include "Hy8402.h"
#include "Hy85156.h"
#include "Hy85056.h"
#include "Hy8601.h"
#include "Hy8005.h"
#include "Hy8403.h" 
#include "Hy8512.h"
#include "Hy8413.h"
#include "Hy8404.h"
#include "Hy8408.h"

/*
 * Revision number of the data structures defined here. The data
 * structures changes in future versions. The users are encouraged
 * to use the VHYTEC_CHECK_VERSION macro to make sure that the version
 * of their conversion module and the version of vhytec.exe are
 * compatible.
 */

#define VHYTEC_H_VERSION 1

typedef struct {
  vista_chix chix;	/* Vista chix. */
  unsigned addr;	/* Hardware address (starting from 0). */
  unsigned id;		/* A unique channel ID in the range
			   from 0 to vhytecEnv.numChannels - 1. */
  int dbDtype;		/* Channel data type (a VDB_DTYPE value). */
  int hwDtype;		/* Data point data type (a VDB_DTYPE value). */
  int appregType;	/* Application register type */
  int thermType;        /* Type for thermocouple conversion */
  int appreg;
  /* More data may be added here in future versions. */

} vhytecChannel_t, VhytecChannel;

typedef VINT8 vhytecBinary_t, VhytecBinary;

typedef union {
  double float64;
  float float32;
  VUINT32 uint32;
  VINT32 int32;
  VUINT16 uint16;
  VINT16 int16;
  VUINT8 uint8;
  VINT8 int8;
  VhytecBinary binary;
} vhytecValue_t, VhytecValue;

/*
 * A single VhytecEnv data structure is allocated per each invocation
 * of the hytec scanner. A pointer to that structure is passed as
 * the first argument to every user written routine.
 */

typedef struct vhytecEnv_s {
  unsigned numChannels;
  VhytecChannel** channel;	/* All numChannels channels. */
  char* commandLineParameter;	/* -userParameter */
  void* user;			/* User specified pointer. */
  int (*checkVersion) (struct vhytecEnv_s*, int);

  /* More data may be added here in future versions. */

} vhytecEnv_t, VhytecEnv;

typedef int (*VhytecRoutine) (VhytecEnv* env,
			       VhytecChannel* chan,
			       VhytecValue* value);
typedef VhytecRoutine vhytecRoutine_t;

#define VHYTEC_CHECK_VERSION(env) \
  (env)->checkVersion (env, VHYTEC_H_VERSION)

/* Data point types. */

#define VHYTEC_TYPE_OUTPUT_DAC 1 
#define VHYTEC_TYPE_INPUT_DO 2
#define VHYTEC_TYPE_OUTPUT_DO 3
#define VHYTEC_TYPE_INPUT_COM 4
#define VHYTEC_TYPE_OUTPUT_MOTOR 5
#define VHYTEC_TYPE_INPUT_ADC 6
#define VHYTEC_TYPE_OUTPUT_COM 7
#define VHYTEC_TYPE_INPUT_MOTOR 8
#define VHYTEC_TYPE_INPUT_U32	9
#define VHYTEC_TYPE_INPUT_APPREG 10
#define VHYTEC_TYPE_OUTPUT_APPREG 11

#define VHYTEC_TYPE_OUTPUT_ADC 23
#define VHYTEC_TYPE_INPUT_DAC 24

#define VHYTEC_TYPE_INPUT_DI 25
#define VHYTEC_TYPE_OUTPUT_DI 29

/* Moved these up to include the Hytec types */
#define VHYTEC_TYPE_INPUT_I1	34 // 1
#define VHYTEC_TYPE_INPUT_I16	35
#define VHYTEC_TYPE_INPUT_U16	36
#define VHYTEC_TYPE_INPUT_F32	37
#define VHYTEC_TYPE_OUTPUT_I1	38
#define VHYTEC_TYPE_OUTPUT_I16	39
#define VHYTEC_TYPE_OUTPUT_U16	40
#define VHYTEC_TYPE_OUTPUT_F32	41
#define VHYTEC_TYPE_INPUT_F64	42
#define VHYTEC_TYPE_OUTPUT_F64	43 // 10
//#define VHYTEC_TYPE_INPUT_U32	44
#define VHYTEC_TYPE_OUTPUT_U32	45
#define VHYTEC_TYPE_INPUT_U64	46
#define VHYTEC_TYPE_OUTPUT_U64	47
#define VHYTEC_TYPE_INPUT_I32	50
#define VHYTEC_TYPE_OUTPUT_I32	51
#define VHYTEC_TYPE_INPUT_I64	52
#define VHYTEC_TYPE_OUTPUT_I64	53

/* CML added these */
#define VHYTEC_TYPE_INPUT_STRING 48
#define VHYTEC_TYPE_OUTPUT_STRING 49
#define VHYTEC_TYPE_INPUT_SDI 13
#define VHYTEC_TYPE_OUTPUT_SDI 14
#define VHYTEC_TYPE_INPUT_HOLDING 15
#define VHYTEC_TYPE_OUTPUT_HOLDING 16
#define VHYTEC_TYPE_INPUT_COILS 17
#define VHYTEC_TYPE_INPUT_HOLDING_32 18
#define VHYTEC_TYPE_OUTPUT_HOLDING_32 19
#define VHYTEC_TYPE_INPUT_HOLDING_64 20
#define VHYTEC_TYPE_OUTPUT_HOLDING_64 21

/* Dispositions. */

#define VHYTEC_DISPOSITION_VARCHIVE 1
#define VHYTEC_DISPOSITION_DATABASE 2
#define VHYTEC_DISPOSITION_BOTH 3

/* Return codes from user routines. */

#define VHYTEC_OK 0
#define VHYTEC_FAILED (-1)

/*                 
		   0= DIF - Differential
		   1 = S/E	- Single Ended                                          
		   2 = PT100	- PT-100 (original)                                 
		   3 = TC	- Thermocouples                                         
		   4 = PTB	- PT-100 with 8212 Transistion Card                     
		   5 = TCB	- Thermocouples with 8212 Transistion Card*/
#define INPUTTYPE_DIF 0
#define INPUTTYPE_SE 1
#define INPUTTYPE_PT100 2
#define INPUTTYPE_TC 3
#define INPUTTYPE_PTB 4
#define INPUTTYPE_TCB 5

#endif

