/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770	*/
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566	*/
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/
/*                                                                              */
/*				IOC9010 PROJECT	--- Hytec IP Card API Functions 				*/
/*							for Windows and Linux Platforms	    				*/
/*                                IP Card Type: 8414                            */
/********************************************************************************/
/*  Source file name      :-      Hy8414.h										*/
/*  Definition header file:		  HYTECIPAPI.h									*/
/*  Platform			  :		  Linux, Windows XP, Windows 2000               */
/*  Initial creation date :-      18-JAN-2012                                   */
/*                                                                              */
/*  Original Developers   :-      Kish Jogia                                    */
/********************************************************************************/
/*                                                                              */
/*  Description :-                                                              */
/*                                                                              */
/*  This header file is designed for Hytec API function library of 8414 IP card.*/
/*  It defines the class prototype, error code, masks and registers of 8414.    */
/*																				*/
/*  The design of this interface is cross platforms, which means the code 		*/
/*	can be used in operating systems like Windows, Linux etc without changes.	*/
/*                                                                              */
/*  The language used is C++ but the final product -- the Dynamic Link library	*/
/*	(dll) or the shared library can be used by either C or C++ applications.	*/
/*																				*/
/*  (C)2012 Hytec Electronics Ltd.												*/
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Modification record :-                                                      */
/*                                                                              */
/*  Date modified |     Person    |   Description of modification | Version     */
/* ---------------|---------------|-------------------------------------------- */
/*    18/01/12    |	  Kish Jogia  |	  Initial copy				  |	0.0.1		*/
/*                |               |                               |             */
/*                                                                              */
/********************************************************************************/
#include "HytecIPAPI.h"

#if PLATFORM == 1

#define HYTECAPI_EXPORTS 1
#ifdef HYTECAPI_EXPORTS
#define HYTECAPI_API __declspec(dllexport)
#else
#define HYTECAPI_API __declspec(dllimport)
#endif
#include "C:\Hytec\IOC9010\IOCGeneric\IOCGeneric\IOCGeneric.h"

#else

#include "IOCGeneric.h"

#endif

//error code definition
#define NOT_8414_IP -8414001
#define GAIN_INVALID_8414 -8414004
#define CLOCKRATE_INVALID_8414 -8414012
#define CHANNEL_INVALID_8414 -8414013
#define Hy8414_NOT_CONFIGURED -8414014
#define Hy8414_DISARM_ERROR -8414015
#define Hy8414_MULTITRIGGER_ERROR -8414016
#define Hy8414_USEINHB_ERROR -8414017
#define Hy8414_ENBINHB_ERROR -8414018
#define Hy8414_ENBHFINT_ERROR -8414019
#define Hy8414_ENBFINT_ERROR -8414020
#define Hy8414_ENBSGLINT_ERROR -8414021
#define Hy8414_TRIGGEREDGE_ERROR -8414022
#define Hy8414_TRIGGERONBOTH_ERROR -8414023

//csr definition
#define CSR_HF 0x0001
#define CSR_F 0x0002
#define CSR_CC 0x0004
#define CSR_MIS 0x0008
#define CSR_MII 0x0010
#define CSR_EII 0x0020
#define CSR_DA 0x0040
#define CSR_1M 0x0080
#define CSR_HE 0x0100
#define CSR_FE 0x0200
#define CSR_EE 0x0400
#define CSR_ET 0x0800
#define CSR_XC 0x1000
#define CSR_ST 0x2000
#define CSR_EX 0x4000
#define CSR_ARM 0x8000

#define EXTCSR_TEN  0x0001          /* Triggered status */
#define EXTCSR_SMI  0x0002          /* Software inhibit control and status. Write 1 to inhibit memory writing; write 0 to allow. Read 1 means hardware inhibit; read 0 means hardware allows */
#define EXTCSR_IS   0x0004          /* Single conversion completed flag */
#define EXTCSR_ISE  0x0008          /* Single conversion completed interrupt enable */
#define EXTCSR_CONT	0x0010          /* =1: multi trigger, =0: trigger once */
#define EXTCSR_R  	0x0040          /* range =0: +/-10V, =1: +/-5V */
#define EXTCSR_TE  	0x1000          /* 0:rising edge trigger, 1:falling edge */
#define EXTCSR_BTE 	0x2000          /* 0: as per TE, 1:both edge trigger */
#define EXTCSR_GAINSHIFT  4    		/* ADC gain setting shift bits */

//signature definition
#define SIGNATURE_8414 0x8414

//register address defines
#define CSR_8414 0x0000
#define POINTER_8414 0x0002
#define NUM_CONVERSIONS_8414 0x0004
#define CLOCK_RATE_8414 0x0006
#define VECTOR_8414 0x0008
#define CSREXT_8414 0x000A
#define FRONT_END_GAIN_8414 0x000C
#define FIRST_CHANNEL_8414 0x0010
#define CHANNEL_MEMORY_BLOCK_8414 0x10000

#define TOP_MEM 0x8000

/******* Interrupt reasons ******/
/* Major reason */
#define INT_REASON_IS 0x01
#define INT_REASON_CC 0x02
#define INT_REASON_MII 0x04
#define INT_REASON_HF 0x08
#define INT_REASON_F 0x10

// This class is exported from the HYTECAPI.dll
#if PLATFORM == 1									//windows
class HYTECAPI_API CHy8414 : public CIOCGeneric
#else											//linux
class CHy8414 : public CIOCGeneric
#endif
{
private:
	long m_ErrorCode;						// global error code holder
	int initIPSlot;								// remember the IP slot when configures
	int initVMESlot;
	char m_vector;
	int m_INTLevel;

public:
	CHy8414(void);
	CHy8414(char *devName);
	CHy8414(int vmeslot, char ipslot);
	~CHy8414(void);

	long GetErrorCode()
	{
		return m_ErrorCode;
	}
	
/*----------------------------------------------------------------------------------*/
/*Function:		Configure 8414 IP card												*/
/*----------------------------------------------------------------------------------*/
/*Function:		Configure 8414 IP card (overloading)								*/
/*----------------------------------------------------------------------------------*/
/*Description:	8414 IP configuration routine										*/
/*				according to the settings.											*/
/*Prototype:	long Hy8414_Configure(int vmeslot, char ipslot, int clockRate)		*/
/*Arguements:	vmeslot = VME slot number											*/
/*				ipslot = IP card slot number 0 ~ 3 for VME, 0 ~ 5 for IOC			*/
/*				mode		-	Operating mode, 0 = Register, 1 = Triggered			*/
/*				clocksource -   0=internal, 1=external								*/
/*				clockrate   -   0-16, clock rate for output if selected. see manual	*/
/*				range		-	ADC range 0 = +/-10V, 1 = +/-5V						*/
/*				vector		-	Interrupt Vector									*/
/*				INTLevel	-	interrupt level. must be 1 ~ 7						*/
/*				callback	-	user callback function								*/
/*				drive		-	user data structure passed to the ISR				*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy8414_Configure(int vmeslot, char ipslot, int mode,
	                    		int clocksource, int clockrate, int range,
								char vector, int INTLevel, void (*callback)(void *), void *drive);

/*----------------------------------------------------------------------------------*/
/*Description:	8414 IP Extension configuration routine								*/
/*				according to the settings.											*/
/*Prototype:	long Hy8414_ExtConfigure(int disarm, int multiTrigger, int useInhb, */
/*					int enbInhbInt,	int enbHFInt, int enbFInt, int enbSingleInt, 	*/
/*					int triggerEdge, int triggerOnBothEdge)							*/
/*Arguements:	disarm = 1: disarm on CC, = 0 on CC doesn't disarm					*/
/*				multiTrigger = 1: continuous trigger, =0: single trigger			*/
/*				useInhb = 1: use inhibit control, 0: don't use inhibit				*/
/*				enbInhbInt = 1: when inhibit is set, interrupt, = 0: no interrupt	*/
/*				enbHFInt = 1: generate interrupt on half full, no interrupt			*/
/*				enbFInt = 1: generate interrupt on full, = 0: no interrupt			*/
/*				enbSingleInt = 1: enable single conversion interrupt, = 0: no int	*/
/*				triggerEdge = 1: trigger on falling edge, = 0: on rising edge		*/
/*				triggerOnBothEdge = 1: trigger on both edge, = 0 as per triggerEdge	*/
/*				Note: any setting above = 2 means it is ignored						*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy8414_ExtConfigure(int disarm, int multiTrigger, int useInhb, int enbInhbInt,	
					int enbHFInt, int enbFInt, int enbSingleInt, int triggerEdge,	
					int triggerOnBothEdge);


/*----------------------------------------------------------------------------------*/
/*Function:		Set Gain															*/
/*----------------------------------------------------------------------------------*/
/*Description:	Set the gain														*/
/*Prototype:	long Hy8414_Gain(int group, int gain)								*/
/*Arguements:	group = 0: first 8 channel, =1: second 8 channel					*/
/*				gain = 0,1,2,3 represent 1,2,4,8 times								*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8414_Gain(int group, int gain);

/*----------------------------------------------------------------------------------*/
/*Function:		Number of Conversion												*/
/*----------------------------------------------------------------------------------*/
/*Description:	Number of Conversion per Trigger									*/
/*Prototype:	long Hy8414_NoConversion(USHORT number)								*/
/*Arguements:	number = the number of conversions									*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8414_NoConversion(USHORT number);

/*----------------------------------------------------------------------------------*/
/*Function:		Set Software Trigger												*/
/*----------------------------------------------------------------------------------*/
/*Description:	Set the software trigger											*/
/*Prototype:	long Hy8414_SoftTrig(void)											*/
/*Arguements:																		*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8414_SoftTrig(void);

/*----------------------------------------------------------------------------------*/
/*Function:		Set Software memory Inhibit											*/
/*----------------------------------------------------------------------------------*/
/*Description:	Set the software memory Inhibit										*/
/*Prototype:	long Hy8414_SoftInhibit(void)										*/
/*Arguements:																		*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8414_SoftInhibit(void);

/*----------------------------------------------------------------------------------*/
/*Function:		Reset Register update bit in ExtCSR									*/
/*----------------------------------------------------------------------------------*/
/*Description:	Reset the Read Register update bit									*/
/*Prototype:	long Hy8414_ResetIS(void)											*/
/*Arguements:																		*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8414_ResetIS(void);

/*----------------------------------------------------------------------------------*/
/*Function:		ARM ADC																*/
/*----------------------------------------------------------------------------------*/
/*Description:	arm ADC at runtime													*/
/*Prototype:	long Hy8414_Arm(void)												*/
/*Arguements:																		*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8414_Arm(void);

/*----------------------------------------------------------------------------------*/
/*Function:		Read memory pointer													*/
/*----------------------------------------------------------------------------------*/
/*Description:	Read memory pointer													*/
/*Prototype:	long Hy8414_ReadPointer(USHORT *pointer)							*/
/*Arguements:	pointer = the pointer of memory										*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8414_ReadPointer(USHORT *pointer);

/*----------------------------------------------------------------------------------*/
/*Function:		Read CSR (mainly for testing)										*/
/*----------------------------------------------------------------------------------*/
/*Description:	Read CSR															*/
/*Prototype:	long Hy8414_ReadCSR(USHORT *csr, USHORT *extcsr)					*/
/*Arguements:	csr = the csr register												*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8414_ReadCSR(USHORT *csr, USHORT *extcsr);

/*----------------------------------------------------------------------------------*/
/*Function:		Read 1 ADC channel of 8414											*/
/*----------------------------------------------------------------------------------*/
/*Description:	Read one channel data from 8414 ADC IP card							*/
/*Prototype:	long Hy8414_Read(int ADC_ch, USHORT *value)							*/
/*Arguements:	ADC_ch = ADC input channel 0 ~ 15									*/
/*				*value = pointer to the 16bit value variable						*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8414_Read(int ADC_ch, USHORT *value);

/*----------------------------------------------------------------------------------*/
/*Function:		Memory Read 1 ADC channel of 8414									*/
/*----------------------------------------------------------------------------------*/
/*Description:	Memory read one channel data from 8414 ADC IP card					*/
/*Prototype:	long Hy8402_Write(int DAC_ch, USHORT *value, int length)			*/
/*Arguements:	ADC_ch = ADC input channel 0 ~ 15									*/
/*				*value = pointer to a 32k word array								*/
/*				length = lenght of array. If length	= 0, read all number of 		*/
/*					conversions														*/
/*				Note: user needs to assure the array is big enough					*/
/*Return:		0 is successfull, negative please see error definition				*/
/*----------------------------------------------------------------------------------*/
	long Hy8414_MemoryRead(int ADC_ch, USHORT *value, int length);

/*----------------------------------------------------------------------------------*/
/*Function:		Enable 8414 interrupt 												*/
/*----------------------------------------------------------------------------------*/
/*Description:	Enable interrupt for 8414 IP card          							*/
/*Prototype:	long Hy8414_EnableInterrupt(USHORT mask)							*/
/*Arguements:	mask - interrupt bit mask											*/
/*					0 0 0 0 0 X X X 0 0 X 0 X 0 0 0 								*/
/*							  | | | 	|	Enable single conversion interrupt		*/
/* 							  | | |		Enable inhibit interrupt					*/
/*							  | | Enable memory full interrupt						*/
/*							  |	Enable memory half full interrupt					*/
/*							  Enable conversion completed interrupt					*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8414_EnableInterrupt(USHORT mask);

/*----------------------------------------------------------------------------------*/
/*Function:		Disable 8414 interrupt             									*/
/*----------------------------------------------------------------------------------*/
/*Description:	Disable interrupt for 8414 IP card          					    */
/*Prototype:	long Hy8414_DisableInterrupt(int reason)							*/
/*Arguements:	reason - integer pointer to the reason of the interrtupt			*/
/*				reason - integer pointer to the reason of the interrtupt			*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8414_DisableInterrupt(int reason);

/*----------------------------------------------------------------------------------*/
/*Function:		Get Interrupt Source												*/
/*----------------------------------------------------------------------------------*/
/*Description:	Get which channel cause the interrtupt and what reason				*/
/*Prototype:	long Hy85156_INTReason(int* port, int *reason)						*/
/*Arguements:	*port = integer pointer to return the interrupt channel 0 ~ 7		*/
/*				*reason = integer pointer to the reason of the interrtupt           */
/*					0 0 0 0 0 0 0 0 0 0 0 X X X X X 								*/
/* 										  | | | | INT_REASON_IS 0x01				*/
/*										  |	| |	INT_REASON_CC 0x02					*/
/*										  | | INT_REASON_MII 0x04					*/
/*										  |	INT_REASON_HF 0x08						*/
/*										  INT_REASON_F 0x10							*/
/*Return:		0 = OK																*/
/*				if error,  negative code in m_errorcode								*/
/*----------------------------------------------------------------------------------*/
	long Hy8414_INTReason(int *reason);

};

#if PLATFORM == 1									//windows
extern "C" HYTECAPI_API CHy8414* CreateCHy8414();
extern "C" HYTECAPI_API void DeleteCHy8414( CHy8414* instance );
#else											//linux
extern "C" CHy8414* CreateCHy8414();
extern "C" void DeleteCHy8414( CHy8414* instance );
#endif

