/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770	*/
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566	*/
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/
/*                                                                              */
/*				IOC9010 PROJECT	--- Hytec IP Card API Functions 				*/
/*							for Windows and Linux Platforms	    				*/
/*                                IP Card Type: 8415                            */
/********************************************************************************/
/*  Source file name      :-      Hy8415.h										*/
/*  Definition header file:		  HYTECIPAPI.h									*/
/*  Platform			  :		  Linux, Windows XP, Windows 2000               */
/*  Initial creation date :-      18-JAN-2012                                   */
/*                                                                              */
/*  Original Developers   :-      Kish Jogia                                    */
/********************************************************************************/
/*                                                                              */
/*  Description :-                                                              */
/*                                                                              */
/*  This header file is designed for Hytec API function library of 8415 IP card.*/
/*  It defines the class prototype, error code, masks and registers of 8415.    */
/*																				*/
/*  The design of this interface is cross platforms, which means the code 		*/
/*	can be used in operating systems like Windows, Linux etc without changes.	*/
/*                                                                              */
/*  The language used is C++ but the final product -- the Dynamic Link library	*/
/*	(dll) or the shared library can be used by either C or C++ applications.	*/
/*																				*/
/*  (C)2012 Hytec Electronics Ltd.												*/
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Modification record :-                                                      */
/*                                                                              */
/*  Date modified |     Person    |   Description of modification | Version     */
/* ---------------|---------------|-------------------------------------------- */
/*    18/01/12    |	  Kish Jogia  |	  Initial copy				  |	0.0.1		*/
/*                |               |                               |             */
/*                                                                              */
/********************************************************************************/
#include "HytecIPAPI.h"

#if PLATFORM == 1

#define HYTECAPI_EXPORTS 1
#ifdef HYTECAPI_EXPORTS
#define HYTECAPI_API __declspec(dllexport)
#else
#define HYTECAPI_API __declspec(dllimport)
#endif
#include "C:\Hytec\IOC9010\IOCGeneric\IOCGeneric\IOCGeneric.h"

#else

#include "IOCGeneric.h"

#endif


//error code definition
#define NOT_8415_IP -8415001
#define CLOCKRATE_INVALID_8415 -8415012
#define CHANNEL_INVALID_8415 -8415013
#define Hy8415_NOT_CONFIGURED -8415014
#define MEMORY_SIZE_INVALID_8415 -8415015
#define DAC_BUSY_8415 -8415016
#define RANGE_INVALID_8415 -8415017
#define INTERRUPT_MASK_INVALID_8415 -8415018
#define FORMAT_ERROR_8415 -8415019
#define MULTITRIGGER_SETTING_ERROR_8415 -8415020
#define REPEAT_PATTERN_ERROR_8415 -8415021
#define BITNUMBER_ERROR_8415 -8415022

/* define bits in the 8415 CSR */
#define CSR_HF 					0x0001          /* STAUTS: indicate finishes of half full updates */
#define CSR_F	 				0x0002          /* STAUTS: indicate finishs of full updates */
#define CSR_CC	 				0x0004          /* STAUTS: indicate conversions complete */
#define CSR_CONT				0x0008          /* Set continuous function generation */
#define CSR_MH 					0x0010          /* STATUS: "1" indicates that memory is inhabited from Strobe line */
#define CSR_1MB 				0x0020          /* using 1MB memory when set or using 2MB when not set */
#define CSR_SSU 				0x0040          /* Writing "1" sets all DAC outputs to zero */
#define CSR_BS    				0x0080          /* STATUS: Flag showing unit is busy shifting data to DAC */
#define CSR_HE     				0x0100          /* Enable interrupt on lower memory has been filled */
#define CSR_FE     				0x0200          /* Enable interrupt on upper memory has been filled */
#define CSR_EE     				0x0400          /* Enable interrupt at the end of number of updates */
#define CSR_ET      			0x0800          /* Enable Strobe line when set to stop updating DAC from memory */
#define CSR_XC      			0x1000          /* 0 => use internal clock, 1 => external clock. */
#define CSR_ST      			0x2000          /* Software trigger. To trigger, do "clear" then "set" */
#define CSR_EX      			0x4000          /* 0 => register mode, 1 => trigger and memory mode. */
#define CSR_ARM     			0x8000          /* ARMing causes the values loaded in to the DAC. cleared afterwards */

/* define extension csr */
#define ACR_PM10V	 			0x0000          /* Range setting: +/-10V */
#define ACR_PM5V	 			0x0001          /* Range setting: +/-5V */
#define ACR_ZT10V	 			0x0002          /* Range setting: 0 -10V */
#define ACR_ZT5V	 			0x0003          /* Range setting: 0 -5V */
#define ACR_BTC	 				0x0004          /* 0 = straight binary, 1 = 2's complement */
#define ACR_MT	 				0x0010          /* Multi trigger mode */
#define ACR_RP					0x0020          /* Repeat pattern in multi trigger mode */
#define ACR_CAL					0x4000          /* only for register mode, set 1 means do not use on board calibration */
#define ACR_EN16				0x8000          /* set 1 to use 16bit DAC, 0 (default) to use 18bit DAC */

//signature definition
#define SIGNATURE_8415 0x8415

//register address defines
#define CSR_8415 				0x0000
#define POINTER_8415			0x0002
#define NUMBER_OF_UPDATE_8415 	0x0004
#define CLOCK_RATE_8415 		0x0006
#define VECTOR_8415 			0x0008
#define CSREXT_8415 			0x000A
#define FIRST_CHANNEL_8415 		0x0010

#define BANKSIZE_8415			0x100000       	/* maximum memory 1M x 16bits */
#define CHANNELBANKSIZE_8415 	0x8000       	/* maximum memory per channel */

#define INT_REASON_HF_8415		0x0001
#define INT_REASON_F_8415		0x0002
#define INT_REASON_CC_8415		0x0004

// This class is exported from the HYTECAPI.dll
#if PLATFORM == 1									//windows
class HYTECAPI_API CHy8415 : public CIOCGeneric
#else											//linux
class CHy8415 : public CIOCGeneric
#endif
{
private:
	long m_ErrorCode;							// global error code holder
	int initIPSlot;								// remember the IP slot when configures
	int initVMESlot;
	int m_bitformat;
	int m_INTLevel;
	char m_vector;

public:
	CHy8415(void);
	CHy8415(char *devName);
	CHy8415(int vmeslot, char ipslot);
	~CHy8415(void);

	long GetErrorCode()
	{
		return m_ErrorCode;
	}
	
/*----------------------------------------------------------------------------------*/
/*Function:		Configure 8415 IP card												*/
/*----------------------------------------------------------------------------------*/
/*Description:	8415 IP configuration routine										*/
/*				according to the settings.											*/
/*Prototype:	long Hy8415_Configure(int vmeslot, char ipslot, int vectornum,		*/
/*				int mode, int clockSource, int clockRate, int inhibit, int range,	*/
/*				char vector, int INTLevel, void (*callback)(void *), void *drive)	*/
/*Arguements:	vmeslot = VME slot number											*/
/*				ipslot = IP card slot number A ~ D for VME, A ~ F for IOC			*/
/*				vectornum = interrupt vector number									*/
/*				doram = 0, output with register, = 1, output with RAM @ClockRate	*/
/*				clockSource = 0, internal, = 1, external							*/
/*				clockRate = 0 ~ 13, clock rate for output if selected. see manual	*/
/*				inhibit = 0, inhibit disabled, = 1, inhibit enabled					*/
/*				range =	ADC range 0 = +/-10V, 1 = +/-5V, 2 = 0-10V, 3 =	0-5V		*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy8415_Configure(int vmeslot, char ipslot, int vectornum, int mode, 
			int clockSource, int clockRate, int inhibit, int range,
			char vector, int INTLevel, void (*callback)(void *), void *drive);

/*----------------------------------------------------------------------------------*/
/*Description:	8415 IP Extension configuration routine								*/
/*				according to the settings.											*/
/*Prototype:	long Hy8415_ExtConfigure(int format, int multiTrigger, 				*/
/*				int repeatPattern,	int bitNumber)									*/
/*Arguements:	format = 1: DAC input 2's complement, = 0 straight binary			*/
/*				multiTrigger = 1: multi trigger, =0: single trigger					*/
/*				repeatPattern = 1: repeat pattern in multi trigger mode, 0: no repeat*/
/*				bitNumber = 1: 16 bit DAC, = 0: 18 bit DAC (default)				*/
/*				Note: any setting above = 2 means it is ignored						*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy8415_ExtConfigure(int format, int multiTrigger, int repeatPattern,	
					int bitNumber);

/*----------------------------------------------------------------------------------*/
/*Function:		Configure 8415 IP card	(Overloading)   							*/
/*----------------------------------------------------------------------------------*/
/*Description:	8415 IP configuration routine. This only change the card address.	*/
/*				One has to be very clear what he/she is doing by using this.		*/
/*				Not recommended to use in normal case. It is designed for EPICS 	*/
/*				device support to have one object to control multiple devices		*/
/*				when the devices are already configured by shell script       		*/
/*Prototype:	long Hy8415_Configure(int vmeslot, int ipslot)        				*/
/*Arguements:	vmeslot = VME slot number											*/
/*				ipslot = IP card slot number 0 ~ 3 for VME, 0 ~ 5 for IOC			*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy8415_Configure(int vmeslot, int ipslot);

/*----------------------------------------------------------------------------------*/
/*Function:		Enable Software Trigger												*/
/*----------------------------------------------------------------------------------*/
/*Description:	Enable the software Trigging of 8415								*/
/*Prototype:	long Hy8415_SoftTrig()												*/
/*Arguements:	vmeslot = VME slot number											*/
/*				ipslot = IP card slot number A ~ D for VME, A ~ F for IOC			*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy8415_SoftTrig(void);

/*----------------------------------------------------------------------------------*/
/*Function:		Enable Continues Function											*/
/*----------------------------------------------------------------------------------*/
/*Description:	Enable continuous function of 8415									*/
/*Prototype:	long Hy8415_ContFunc()												*/
/*Arguements:	enable = 1 enable continuous update, = 0 only output once			*/
/*				ipslot = IP card slot number A ~ D for VME, A ~ F for IOC			*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy8415_ContFunc(int enable);

/*----------------------------------------------------------------------------------*/
/*Function:		Write One Channel													*/
/*----------------------------------------------------------------------------------*/
/*Description:	Output data to a DAC channel of 8415								*/
/*Prototype:	long Hy8415_Write(USHORT DAC_ch, ULONG value)						*/
/*Arguements:	ADC_ch = DAC output channel 0 ~ 15									*/
/*				value = unsigned 18bit value										*/
/*Return:		0 is successfull, negative please see error definition				*/
/*----------------------------------------------------------------------------------*/
	long Hy8415_Write(int DAC_ch, ULONG value);

/*----------------------------------------------------------------------------------*/
/*Function:		Read One Channel													*/
/*----------------------------------------------------------------------------------*/
/*Description:	Read data from a DAC channel of 8415								*/
/*Prototype:	long Hy8415_Read(USHORT DAC_ch, ULONG *value)						*/
/*Arguements:	ADC_ch = DAC output channel 0 ~ 15									*/
/*				*value = pointer to receive the 18bit value							*/
/*Return:		0 is successfull, negative please see error definition				*/
/*----------------------------------------------------------------------------------*/
	long Hy8415_Read(int DAC_ch, ULONG *value);

/*----------------------------------------------------------------------------------*/
/*Function:		Write Waveform Data	(Overloading)									*/
/*----------------------------------------------------------------------------------*/
/*Description:	Output the whole waveform data (64k byte) to a DAC channel memory area	*/
/*Prototype:	long Hy8415_Write(int DAC_ch, USHORT *value)						*/
/*Arguements:	ADC_ch = DAC output channel 0 ~ 15									*/
/*				*value = pointer to a 32k word array								*/
/*				lenght = lenght of array											*/
/*Return:		0 is successfull, negative please see error definition				*/
/*----------------------------------------------------------------------------------*/
	long Hy8415_Write(int DAC_ch, ULONG *value, int length);

/*----------------------------------------------------------------------------------*/
/*Function:		Enable 8415 interrupt 												*/
/*----------------------------------------------------------------------------------*/
/*Description:	Enable interrupt for 8415 IP card          							*/
/*Prototype:	long Hy8415_EnableInterrupt(USHORT mask)							*/
/*Arguements:	mask - interrupt bit mask											*/
/*					0 0 0 0 0 X X X 0 0 0 0 0 0 0 0 								*/
/* 							  | | |													*/
/*							  | | Enable lower memory filled full interrupt			*/
/*							  |	Enable upper memory filled full interrupt			*/
/*							  Enable conversion completed interrupt					*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8415_EnableInterrupt(USHORT mask);

/*----------------------------------------------------------------------------------*/
/*Function:		Disable 8415 interrupt             									*/
/*----------------------------------------------------------------------------------*/
/*Description:	Disable interrupt for 8415 IP card          					    */
/*Prototype:	long Hy8415_DisableInterrupt(int reason)							*/
/*Arguements:	reason - integer pointer to the reason of the interrtupt			*/
/*					0 0 0 0 0 0 0 0 0 0 0 0 0 X X X 								*/
/* 										      | | INT_REASON_HF 0x01				*/
/*										   	  |	INT_REASON_F 0x02					*/
/*										      INT_REASON_CC 0x04					*/
//*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8415_DisableInterrupt(int reason);

/*----------------------------------------------------------------------------------*/
/*Function:		Get Interrupt Source												*/
/*----------------------------------------------------------------------------------*/
/*Description:	Get which channel cause the interrtupt and what reason				*/
/*Prototype:	long Hy8415_INTReason(int *reason)									*/
/*Arguements:	*port = integer pointer to return the interrupt channel 0 ~ 7		*/
/*				*reason = integer pointer to the reason of the interrtupt           */
/*					0 0 0 0 0 0 0 0 0 0 0 0 0 X X X 								*/
/* 										      | | INT_REASON_HF 0x01				*/
/*										   	  |	INT_REASON_F 0x02					*/
/*										      INT_REASON_CC 0x04					*/
/*Return:		0 = OK																*/
/*				if error,  negative code in m_errorcode								*/
/*----------------------------------------------------------------------------------*/
	long Hy8415_INTReason(int *reason);


};
#if PLATFORM == 1									//windows
extern "C" HYTECAPI_API CHy8415* CreateCHy8415();
extern "C" HYTECAPI_API void DeleteCHy8415( CHy8415* instance );
#else											//linux
extern "C" CHy8415* CreateCHy8415();
extern "C" void DeleteCHy8415( CHy8415* instance );
#endif

