/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770	*/
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566	*/
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/
/*                                                                              */
/*				IOC9010 PROJECT	--- Hytec IP Card API Functions 				*/
/*							for Windows and Linux Platforms	    				*/
/*                                IP Card Type: 8510                            */
/********************************************************************************/
/*  Source file name      :-      Hy8511.h	 									*/
/*  Definition header file:		  HYTECIPAPI.h									*/
/*  Platform			  :		  Linux, Windows XP, Windows 2000               */
/*  Initial creation date :-      03-May-2012                                   */
/*                                                                              */
/*  Original Developers   :-      Mark Woodward                                 */
/********************************************************************************/
/*                                                                              */
/*  Description :-                                                              */
/*                                                                              */
/*  This header file is designed for Hytec API function library of 8511 digital */
/*	input IP card. It defines the class prototype, error code, masks and 		*/
/*	registers of 8511 and the associated 8960.															*/
/*																				*/
/*  The design of this interface is cross platforms, which means the code 		*/
/*	can be used in operating systems like Windows, Linux etc without changes.	*/
/*                                                                              */
/*  The language used is C++ but the final product -- the Dynamic Link library	*/
/*	(dll) or the shared library can be used by either C or C++ applications.	*/
/*																				*/
/*  (C)2011 Hytec Electronics Ltd.												*/
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Modification record :-                                                      */
/*                                                                              */
/*  Date modified |     Person    |   Description of modification | Version     */
/* ---------------|---------------|-------------------------------------------- */
/*    03/05/12    |	Mark Woodward |	  Initial copy				  |	0.0.1		*/
/*    19/01/15    |	Mark Woodward |	  Added O/C timing modes	  |	0.0.2		*/
/*                |               |                                             */
/*                                                                              */
/********************************************************************************/
#include "HytecIPAPI.h"
#if PLATFORM == 1

#define HYTECAPI_EXPORTS 1
#ifdef HYTECAPI_EXPORTS
#define HYTECAPI_API __declspec(dllexport)
#else
#define HYTECAPI_API __declspec(dllimport)
#endif
#include "IOCGeneric.h"

#else

#include "IOCGeneric.h"

#endif

//error code definition
#define NOT_8511_IP -8511010
#define Hy8511_NOT_CONFIGURED -8511011
#define Hy8511_NO_8960 -8511012
#define Hy8960_CARD_INVALID -8960013
#define Hy8960_TIMING_INVALID -8960014


//signature definition
#define SIGNATURE_8511 0x8511

//register address defines
#define CSR_8511 0x0000
#define INTERRUPT_VECTOR_8511 0x0002
#define WD_8511 0x0004

#define MODEL_8960 0x0020
#define REVISION_8960 0x0022

#define GAIN_Z_1_16_8960 0x0040
#define GAIN_Z_17_20_8960 0x0042
#define GAIN_Y_1_16_8960 0x0044
#define GAIN_Y_17_20_8960 0x0046
#define CSR_8960 0x0048

#define SAM_TIME_CNT_8960 0x004A
#define FLY_TIME_CNT_8960 0x004C
#define IO_TIME_CNT_8960 0x004E
#define SWD_TIME_CNT_8960 0x0050
#define ADC_TIME_CNT_8960 0x0052
#define SCANRATE_8960 0x0054
#define DEL_FC20_8960 0x0056
#define SCR_8960 0x0058
#define D00_SAM_TIME_CNT_8960 0x0060
#define D01_SAM_TIME_CNT_8960 0x0062
#define D10_SAM_TIME_CNT_8960 0x0064
#define D11_SAM_TIME_CNT_8960 0x0066
#define D00_FLY_TIME_CNT_8960 0x0068
#define D01_FLY_TIME_CNT_8960 0x006A
#define D10_FLY_TIME_CNT_8960 0x006C
#define D11_FLY_TIME_CNT_8960 0x006E
#define D00_IO_TIME_CNT_8960 0x0070
#define D01_IO_TIME_CNT_8960 0x0072
#define D10_IO_TIME_CNT_8960 0x0074
#define D11_IO_TIME_CNT_8960 0x0076
#define D00_SWD_TIME_CNT_8960 0x0078
#define D01_SWD_TIME_CNT_8960 0x007A
#define D10_SWD_TIME_CNT_8960 0x007C
#define D11_SWD_TIME_CNT_8960 0x007E
#define OC_SAM_TIME_CNT_8960 0x0054
#define OC_IO_FLY_TIME_CNT_8960 0x005A
#define OC_SWD1_TIME_CNT_8960 0x005C
#define OC_SWD2_TIME_CNT_8960 0x005E


//memory address defines
//#define READ_8511 0x0000

//bit definitions
#define CON_8511 0x0004
#define ENABLE_WD_8511 0x8000
#define ENABLE_OC_8960 0x0800
#define ENABLE_SCAN_8960 0x0010
#define CONERR_8960 0x0080
#define MPXERR_8960 0x0100
#define IE_8960 0x0002
#define MAINS_CYCLE_8960 0x0004
#define EMS_8960 0x0008
#define CB1_8960 0x1000
#define CB2_8960 0x2000
#define CB3_8960 0x4000
#define OCD_8960 0x8000

// This class is exported from the HYTECAPI.dll
#if PLATFORM == 1									//windows
class HYTECAPI_API CHy8511 : public CIOCGeneric
#else											//linux
class CHy8511 : public CIOCGeneric
#endif
{
private:
	long m_ErrorCode;							// global error code holder
	int initIPSlot;								// remember the IP slot when configures
	int initVMESlot;
	char m_vector;
	int m_INTLevel;

public:
	CHy8511(void);
	CHy8511(int vmeslot, char ipslot);
	~CHy8511(void);

	long GetErrorCode()
	{
		return m_ErrorCode;
	}
	
/*----------------------------------------------------------------------------------*/
/*Function:		Configure 8511 / 8960 IP card												*/
/*----------------------------------------------------------------------------------*/
/*Description:	8511 IP configuration routine										*/
/*				according to the settings.											*/
/*Prototype:	long Hy8511_Configure(int vmeslot, char ipslot, int vectornum)		*/
/*																					*/
/*Arguements:	vmeslot = VME slot number											*/
/*				ipslot = IP card slot number 0 ~ 3 for VME, 0 ~ 5 for IOC			*/
/*				vectornum = interrupt vector number									*/
/*Return:		status code, 0 = successful. negative = errors						*/
/*----------------------------------------------------------------------------------*/
	long Hy8511_Configure(int vmeslot, char ipslot, USHORT vectornum);

/*----------------------------------------------------------------------------------*/
/*Function:		Trigger Scan														*/
/*----------------------------------------------------------------------------------*/
/*Description:	Writes the card address and triggers the scan						*/
/*Prototype:	long Hy8960_TriggerScan(USHORT card)								*/
/*Arguements:	card = card to trigger (0-15)										*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8960_TriggerScan(USHORT card);

/*----------------------------------------------------------------------------------*/
/*Function:		Read 8511 CSR Register												*/
/*----------------------------------------------------------------------------------*/
/*Description:	Read the 8511 CSR register											*/
/*Prototype:	long Hy8511_ReadCSR(void)											*/
/*Arguements:	*value = pointer to the location to receive data					*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8511_ReadCSR(USHORT *value);

/*----------------------------------------------------------------------------------*/
/*Function:		Write 8511 CSR Register												*/
/*----------------------------------------------------------------------------------*/
/*Description:	Write the 8511 CSR register											*/
/*Prototype:	long Hy8511_WriteCSR(void)											*/
/*Arguements:	value = Write data													*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8511_WriteCSR(USHORT value);

/*----------------------------------------------------------------------------------*/
/*Function:		Read 8511 WD Register												*/
/*----------------------------------------------------------------------------------*/
/*Description:	Read the 8511 WD register											*/
/*Prototype:	long Hy8511_ReadWD(void)											*/
/*Arguements:	*value = pointer to the location to receive data					*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8511_ReadWD(USHORT *value);

/*----------------------------------------------------------------------------------*/
/*Function:		Write 8511 WD Register												*/
/*----------------------------------------------------------------------------------*/
/*Description:	Write the 8511 WD register											*/
/*Prototype:	long Hy8511_WriteWD(void)											*/
/*Arguements:	*value = pointer to the location to receive data					*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8511_SetWD(USHORT value);

/*----------------------------------------------------------------------------------*/
/*Function:		Read 8960 CSR Register												*/
/*----------------------------------------------------------------------------------*/
/*Description:	Read the 8960 CSR register											*/
/*Prototype:	long Hy8960_ReadCSR(void)											*/
/*Arguements:	*value = pointer to the location to receive data					*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8960_ReadCSR(USHORT *value);

/*----------------------------------------------------------------------------------*/
/*Function:		Read 8960 Scan Control Register												*/
/*----------------------------------------------------------------------------------*/
/*Description:	Read the 8960 SCR register											*/
/*Prototype:	long Hy8960_ReadSCR(void)											*/
/*Arguements:	*value = pointer to the location to receive data					*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8960_ReadSCR(USHORT *value);

/*----------------------------------------------------------------------------------*/
/*Function:		Set the 8960 Timing Registers										*/
/*----------------------------------------------------------------------------------*/
/*Description:	Set up block of 4 timing registers on 8960							*/
/*Prototype:	long Hy8960_Timing(USHORT block, USHORT *data)						*/
/*Arguements:	block - 0 - 3  - which block to update								*/
/*				data - address of 4 element array with timing data					*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8960_Timing(USHORT block, USHORT *data);

/*----------------------------------------------------------------------------------*/
/*Function:		Set DEL20 register													*/
/*----------------------------------------------------------------------------------*/
/*Description:	Sets the DEL20 register												*/
/*Prototype:	long Hy8960_SetDel20(USHORT data)									*/
/*Arguements:	data = data to overwrite register									*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8960_SetDel20(USHORT data);

/*----------------------------------------------------------------------------------*/
/*Function:		Set gain Z															*/
/*----------------------------------------------------------------------------------*/
/*Description:	Sets the Gain Z register											*/
/*Prototype:	long Hy8960_TriggerScan(long data)									*/
/*Arguements:	data = data to overwrite register - 20 bits used					*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8960_SetGainZ(long data);

/*----------------------------------------------------------------------------------*/
/*Function:		Set gain Y															*/
/*----------------------------------------------------------------------------------*/
/*Description:	Sets the Gain Y register											*/
/*Prototype:	long Hy8960_TriggerScan(long data)									*/
/*Arguements:	data = data to overwrite register - 20 bits used					*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
	long Hy8960_SetGainY(long data);

/*----------------------------------------------------------------------------------*/
/*Function:		Read 8960 Revision Register											*/
/*----------------------------------------------------------------------------------*/
/*Description:	Read the 8960 Revision register										*/
/*Prototype:	long Hy8960_Revision(USHORT *value)									*/
/*Arguements:	*value - pointer to location to receive data						*/
/*Return:		0 is successfull, negative please see error define					*/
/*----------------------------------------------------------------------------------*/
long Hy8960_Revision(USHORT *value);

};

#if PLATFORM == 1									//windows
extern "C" HYTECAPI_API CHy8511* CreateCHy8511();
extern "C" HYTECAPI_API CHy8511* CreateCHy8511A(int vmeslot, char ipslot);
extern "C" HYTECAPI_API void DeleteCHy8511( CHy8511* instance );
#else											//linux
extern "C" CHy8511* CreateCHy8511();
extern "C" CHy8511* CreateCHy8511A(int vmeslot, char ipslot);
extern "C" void DeleteCHy8511( CHy8511* instance );
#endif

